/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef ASM_CONFIGWIDGET
#define ASM_CONFIGWIDGET

#include <qcombobox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <klocale.h>
#include <klistview.h>
#include <kdialogbase.h>

#include "ConfigWidget.h"

class DeviceChooserDialog : public KDialogBase
{
  Q_OBJECT
public:
  DeviceChooserDialog(const QString &device, QWidget *parent);

  QString device() const;

private slots:
  void listDoubleClicked(QListViewItem *item);

private:
  KListView *_listView;
};

class DeviceChooser : public QWidget
{
  Q_OBJECT
public:
  DeviceChooser(bool withAuto, QWidget *parent);
  DeviceChooser(const QString &device, bool withAuto, QWidget *parent);

  QString device() const;
  static QString label(const QString &device);

public slots:
  void setDevice(const QString &device);

signals:
  void changed();

private slots:
  void chooseDevice();
  void autoChanged(bool on);

private:
  QCheckBox   *_auto;
  QPushButton *_button;
  QString      _device;

  void init(bool withAuto);
};

class AsmConfigWidget : public ConfigWidget
{
  Q_OBJECT
public:
  AsmConfigWidget();

  virtual QString configName() const { return i18n("General"); }
  virtual QString title() const { return i18n("General"); }
  virtual QPixmap icon() const;

public slots:
  virtual void doApply();

private:
  // assembler configuration
  DeviceChooser *_device;
  QComboBox *ff ; // file formats
  QComboBox *ra ; // Radix
  QComboBox *wa ; // Warning levels
  QLineEdit *id ; // Include directories
  QLineEdit *oo ; // Other options

  // deassembler configuratioin
  QLineEdit *doo ; // Other options
  QCheckBox *dshort ; // short linsting needed

  enum { Nb_Radix = 2 };
  static const char *RADIX[Nb_Radix];
  enum { Nb_WarningLevels = 3 };
  static const char *WARNING_LEVELS[Nb_WarningLevels];
};

#endif
