/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "BinWordEditor.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qcombobox.h>

BinWordEditor::BinWordEditor(Memory &memory, uint address, QWidget *parent)
  : MemorySpaceEditor(memory, MemoryConfig, false, parent), _address(address)
{
  QGridLayout *grid = new QGridLayout(_top);
  grid->setColStretch(2, 1);

  uint line = 0;
  //qDebug("BinWordEditor %i %i", address, memory.config(address).count());
  for (uint k=0; k<memory.config(address).count(); k++) {
    const Memory::ConfigBit &bit = memory.config(address)[k];
    //qDebug("  %i %s", k, bit.mask_name.latin1());
    if ( _combos.find(bit.mask_name)==_combos.end() ) {
      if (bit.readOnly) {
        _combos[bit.mask_name] = 0;
        continue;
      }
      QLabel *label = new QLabel(bit.mask_name, this);
      grid->addWidget(label, line, 0);
      QComboBox *combo = new QComboBox(this);
      _combos[bit.mask_name] = combo;
      connect(combo, SIGNAL(activated(int)), SLOT(slotModified()));
      grid->addWidget(combo, line, 1);
      label = new QLabel(bit.comment, this);
      grid->addWidget(label, line, 2);
      line++;
    }
    if (_combos[bit.mask_name])
      _combos[bit.mask_name]->insertItem(bit.value_name);
  }
}

void BinWordEditor::setReadWrite(bool readWrite)
{
  QMap<QString, QComboBox *>::const_iterator it = _combos.begin();
  for (; it!=_combos.end(); ++it)
    if ( it.data() ) it.data()->setEnabled(readWrite);
}

void BinWordEditor::slotModified()
{
  uint &w = *_memory.dataAtAddress(MemoryConfig, _address);
  for (uint k=0; k<_memory.config(_address).count(); k++) {
    const Memory::ConfigBit &bit = _memory.config(_address)[k];
    QComboBox *combo = _combos[bit.mask_name];
    if ( combo && combo->currentText()==bit.value_name ) {
      w &= ~bit.mask; // clear masked bits
      w |= bit.value; // set value
    }
  }
  emit modified();
}

void BinWordEditor::updateDisplay()
{
  const uint &w = *_memory.dataAtAddress(MemoryConfig, _address);
  for (uint k=0; k<_memory.config(_address).count(); k++) {
    const Memory::ConfigBit &bit = _memory.config(_address)[k];
    if ( w & bit.value ) {
      QComboBox *combo = _combos[bit.mask_name];
      if (combo) combo->setCurrentText(bit.value_name);
    }
  }
}
