/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "BinWordsEditor.h"

#include <qtabwidget.h>
#include <qlayout.h>
#include <qvgroupbox.h>

#include "memory.h"
#include "BinWordEditor.h"
#include "microchip/misc.h"

BinWordsEditor::BinWordsEditor(Memory &memory, QWidget *parent)
  : MemorySpaceEditor(memory, MemoryConfig, true, parent, "configuration_bits_editor")
{
  QHBoxLayout *hb = new QHBoxLayout(_top);

  QTabWidget *tabw = 0;
  uint n = memory.length(MemoryConfig);
  if ( n>1 ) {
    tabw = new QTabWidget(this);
    hb->addWidget(tabw);
  }

  for(uint i=0; i<n; ++i) {
    uint address = memory.start(MemoryConfig) + i;
    qDebug("BinWordsEditor: %s %i", toHex(address, 4).data(), memory.config(address).count());
    if ( memory.config(address).count()==0 ) continue;
    QWidget *page = 0;
    if ( n>1 ) {
      page = new QWidget(tabw);
      tabw->addTab(page, memory.config(address).name);
    } else {
      page = new QGroupBox(this);
      hb->addWidget(page);
    }
    QVBoxLayout *vbox = new QVBoxLayout(page, 10, 10);
    QHBoxLayout *hbox = new QHBoxLayout(vbox);
    MemoryDisplayerBase *mdb = new MemoryDisplayerBase(page, memory, MemoryConfig, 1, 1, i, 1);
    connect(mdb, SIGNAL(modified()), SIGNAL(modified()));
    _editors.append(mdb);
    hbox->addWidget(mdb);
    hbox->addStretch(1);
    hbox = new QHBoxLayout(vbox);
    BinWordEditor *we = new BinWordEditor(memory, address, page);
    connect(we, SIGNAL(modified()), SIGNAL(modified()));
    connect(we, SIGNAL(modified()), mdb, SLOT(updateDisplay()));
    connect(mdb, SIGNAL(modified()), we, SLOT(updateDisplay()));
    _editors.append(we);
    hbox->addWidget(we);
    hbox->addStretch(1);
    vbox->addStretch(1);
  }
}

void BinWordsEditor::setReadWrite(bool readWrite)
{
  for (uint i=0; i<_editors.count(); i++)
    _editors[i]->setReadWrite(readWrite);
}

void BinWordsEditor::updateDisplay()
{
  for (uint i=0; i<_editors.count(); i++)
    _editors[i]->updateDisplay();
}
