/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "Config.h"

#include <kapplication.h>
#include <kconfig.h>

#include "device_list.h"

void Config::getAsmConfig(QString &hex, QString &include, QString& radix,
	QString &warning, QString &other)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("asm") ;
    hex =  conf->readEntry( "hexfmt", "inhx32" ) ;
    include =   conf->readEntry( "include", "$(SRCPATH)" ) ;
    radix =  conf->readEntry( "radix", "dec" ) ;
		warning =   conf->readEntry( "warning", "2" ) ;
    other =  conf->readEntry("others", "" ) ;
}

void Config::setAsmConfig(const  QString &hexformat,const  QString &include,const  QString& radix,
const 	QString &warnlevel,const  QString &others)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("asm") ;
    conf->writeEntry("hexfmt", hexformat);
    conf->writeEntry("include", include) ;
    conf->writeEntry("radix", radix) ;
    conf->writeEntry("warning", warnlevel) ;
    conf->writeEntry("others", others) ;
}

void Config::getDAsmConfig(bool& shortlist, QString& other)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("dasm") ;
    shortlist =  conf->readBoolEntry( "shortlist", true ) ;
    other =  conf->readEntry("others", "" ) ;
}
void Config::setDAsmConfig(bool shortlist, const QString& other)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("dasm") ;
    conf->writeEntry("shortlist", shortlist ) ;
    conf->writeEntry("others", other) ;
    conf->sync() ;
}

void Config::getGlobal(QString &version)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("global-settings") ;
    version  = conf->readEntry( "version", "<Unknown>" ) ;
}

void Config::setGlobal(const QString& version)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("global-settings") ;
    conf->writeEntry("version", version) ;
}

void Config::setOpened(const PURL& project, const PURL::List& files)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("opened-files") ;
    conf->writeEntry("project", project.filepath());
    uint i = 0;
    PURL::List::const_iterator it = files.begin();
    QString f = "file%1";
    for (; it!=files.end(); ++it) {
      conf->writeEntry(f.arg(i), (*it).filepath());
      i++;
    }
    conf->writeEntry(f.arg(i), QString::null);  // empty entry == end of file list
}

void Config::getOpened(PURL& project, PURL::List& files)
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("opened-files") ;
    project = PURL(conf->readEntry("project", QString::null));
    files.clear() ;
    QString file, name = "file%1" ;
    int i = 0;
    for (;;) {
      QString file = conf->readEntry(name.arg(i), QString::null);
      if ( file.isEmpty() ) break;
      PURL url(file);
      files += url;
      i++;
    }
}

void Config::writeProgrammerType(DeviceUser::Type type)
{
  KConfig *conf = kapp->config();
  conf->setGroup("");
  conf->writeEntry("programmer", type);
}
DeviceUser::Type Config::programmerType()
{
  KConfig *conf = kapp->config();
  conf->setGroup("");
  uint i = conf->readUnsignedNumEntry("programmer", DeviceUser::ICD2Programmer);
  if ( !(DeviceUser::DATA[i].properties & DeviceUser::Programmer) )
    return DeviceUser::defaultType(DeviceUser::Programmer);
  return DeviceUser::Type(i);
}

void Config::writeDefaultDevice(const QString &name)
{
  KConfig *conf = kapp->config();
  conf->setGroup("");
  conf->writeEntry("device", name);
}
QString Config::defaultDevice()
{
  KConfig *conf = kapp->config();
  conf->setGroup("");
  return DeviceList::checkName(conf->readEntry("device"));
}

void Config::writeShowDebugLog(bool show)
{
  KConfig *conf = kapp->config();
  conf->setGroup("");
  conf->writeEntry("show_debug_log", show);
}
bool Config::showDebugLog()
{
  KConfig *conf = kapp->config();
  conf->setGroup("");
  return conf->readBoolEntry("show_debug_log", false);
}
