/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2004 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "ConfigCenter.h"

#include "ConfigWidget.h"
#include "AsmConfigWidget.h"
#include "PrgConfigWidget.h"

#include <qlistview.h>
#include <qwidgetstack.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qmainwindow.h>
#include <qtoolbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qfont.h>
#include <qcolor.h>
#include <qvaluelist.h>
#include <qlayout.h>

#include <klocale.h>

ConfigCenter::ConfigCenter(QWidget *parent)
  : KDialogBase(parent, "configure_dialog", true, i18n("Configure Piclab"), Ok|Cancel, Cancel)
{
    QWidget *cw = new QWidget(this);
    setMainWidget(cw);
    QHBoxLayout *hbox = new QHBoxLayout(cw, 10, 10);

    // add control tree
    confView =    new  QListView(cw) ;
    hbox->addWidget(confView);
    int firstcol = confView->addColumn(i18n("Configure"),160) ;
    confView->setColumnWidthMode(firstcol,QListView::Manual) ;
    confView->setMinimumWidth(165) ;
    confView->setRootIsDecorated(true) ;
    confView->setItemMargin(4) ;
    confView->header()->hide() ;
    confView->setSelectionMode( QListView::NoSelection ) ;
    connect( confView , SIGNAL(currentChanged(QListViewItem *)), this, SLOT(raiseConfigWidget(QListViewItem *))) ;

    QVBoxLayout *vbox = new QVBoxLayout(10);
    hbox->addLayout(vbox, 1);;

    hbox = new QHBoxLayout(vbox);
    image = new QLabel(cw) ;
    hbox->addWidget(image);
    title = new QLabel(cw) ;
    hbox->addWidget(title);
    title->setAlignment(Qt::AlignCenter) ;
    QFont f = title->font() ;
    f.setPointSize(14) ;
    title->setFont(f) ;
    title->setPaletteForegroundColor(QColor(0x7D,0x70,0x70)) ;

    wstack = new QWidgetStack(cw);
    vbox->addWidget(wstack, 1);

    ConfigWidget *first = new AsmConfigWidget();
    plugConfigWidget(first);
    title->setText(first->title());
    image->setPixmap(first->icon());
    plugConfigWidget(new ProgConfigWidget());
}

void ConfigCenter::plugConfigWidget(ConfigWidget *cw)
{
 int id = wstack->addWidget(cw);
 QListViewItem *conf = new QListViewItem(confView, cw->title(), QString::number(id));
 conf->setPixmap(0, cw->icon());
}

void ConfigCenter::slotOk()
{
  ConfigWidget *ac = static_cast<ConfigWidget *>(wstack->visibleWidget());
  ac->doApply();
  slotClose();
}

void ConfigCenter::raiseConfigWidget(QListViewItem *item)
{
  int id = item->text(1).toInt();
  wstack->raiseWidget(id);
  ConfigWidget *ac = static_cast<ConfigWidget *>(wstack->visibleWidget());
  ac->doApply();
  title->setText(ac->title());
  image->setPixmap(ac->icon());
}
