/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef EDITOR_MANAGER_H
#define EDITOR_MANAGER_H

#include <qtabwidget.h>
#include <qtabbar.h>
#include <qptrlist.h>
#include <qevent.h>

#include <kaction.h>

#include "FNU.h"
#include "editor.h"
class PikMain;

// Regroupe toutes les commandes pour piloter un editeur
// KBEditor, et regle les pb d'interaction entre icone/menu et editeur
class EditorManager : public QTabWidget
{
  Q_OBJECT
public:
  EditorManager(PikMain *main, QWidget *parent);
  virtual ~EditorManager();

  PURL currentFile() const;
  PURL::List files() const;
  uint nbEditors() const { return _editors.count(); }
  Editor *createEditor(const PURL &url);
  void addEditor(Editor *e);
  Editor *currentEditor() const { return _current; }
  Editor *findEditor(const PURL &file);
  Editor *findEditor(const QString &name);
  Editor *findOrCreateEditor(const PURL &url);
  Editor *switchToEditor(const PURL &url);
  void changeToEditor(Editor *e);
  bool closeEditor(const PURL &url);
  bool closeEditor(Editor *e);
  void updateTitle(Editor *e);

public slots:
  void slotDropEvent(QDropEvent *e);
  void newFile();
  bool openFile();
  bool openFile(const KURL &url);
  bool simpleOpenFile(const PURL &url);
  void saveAllFiles();
  bool closeCurrentEditor() { return closeEditor(_current); }
  bool closeAllFiles();
  bool closeAllOthers();

signals:
  void guiChanged();
  void statusChanged(const QString &);

private:
  void connectEditor(Editor *editor);
  void disconnectEditor(Editor *editor);
  void enableActions(bool enable);
  virtual bool eventFilter(QObject *watched, QEvent *e);

private slots:
  void changeToEditor(QWidget *w) { changeToEditor(static_cast<Editor *>(w)); }

private:
  PikMain *_main;
  Editor  *_current;
  QValueList<Editor *> _editors;
};

#endif
