/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "HexValueEditor.h"

#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qapplication.h>

#include "microchip/misc.h"

HexValueEditor::HexValueEditor(QWidget *parent, unsigned int *data,
                               unsigned int mask, uint nbChars)
  : QLineEdit(parent), _readWrite(true), _nbChars(nbChars), _data(data), _mask(mask)
{
  connect(this, SIGNAL(textChanged(const QString &)), SLOT(tryUpdate(const QString &)));
  connect(this, SIGNAL(lostFocus()), SLOT(slotLostFocus()));
  setNewAdress(data);
  setFrame(false);
  setMaxLength(nbChars);
}

void HexValueEditor::tryUpdate(const QString &txt)
{
    if ( _data==0 || txt.isEmpty() ) return;
    for (uint i=0; i<txt.length(); ++i) {
      if ( !isxdigit(txt[i].unicode()) ) {
        setText(txt.left(i));
        QApplication::beep();
        return;
      }
    }
    *_data = txt.toUInt(0, 16) & _mask;
    emit modified();
}

void HexValueEditor::slotLostFocus()
{
  if ( _data==0 ) return;
  blockSignals(true);
  setText(toHex(*_data, _nbChars));
  blockSignals(false);
}

QSize  HexValueEditor::sizeHint() const
{
  return QSize(maxHexCharWidth(font()) * (_nbChars+1), fontMetrics().height());
}

QSize  HexValueEditor::minimumSizeHint() const
{
  return QSize(maxHexCharWidth(font()) * (_nbChars+1), fontMetrics().height());
}

void HexValueEditor::setNewAdress(uint *data)
{
  blockSignals(true);
  _data = data;
  setEnabled(data && _readWrite);
  if (data) setText(toHex(*_data & _mask, _nbChars));
  else setText(QString::null);
  blockSignals(false);
}

void HexValueEditor::setColor(const QColor& color)
{
  setPaletteBackgroundColor(color) ;
}

void HexValueEditor::setReadWrite(bool readWrite)
{
  _readWrite = readWrite;
  setEnabled(_data && readWrite);
}
