/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef KBEDITOR_H
#define KBEDITOR_H

#include <qvbox.h>
#include <qsplitter.h>
#include <qlabel.h>
#include <qevent.h>
#include <qstringlist.h>

#include <kparts/mainwindow.h>
#include <kparts/event.h>
#include <kate/view.h>
#include <kate/document.h>
#include <klibloader.h>
#include <kstdaction.h>
#include <kpopupmenu.h>

#include "editor.h"

// KBEditor , a Kate Based Editor Widget
class KBEditor : public Editor
{
    Q_OBJECT
public://private:
    Kate::Document *doc ;
    QSplitter *split ;
    Kate::View *view ;
    QStringList descript;
    // void populateList();
    bool old_read_write_state  ;
    int selected_line ;
    /** HL scheme number for XPicAsm sources */
    uint picHLNumber ;

public:
  KBEditor(const PURL &url, QWidget *parent);
  virtual  ~KBEditor();

  virtual int capabilities() const { return Common | Selection | Search |
                                            Line | Print | View | UndoRedo; }
  virtual uint value(ValueType type) const;
  virtual PURL::FileType fileType() const { return url().fileType(); }
  virtual bool isModified() const;
  virtual bool isReadWrite() const;
  virtual void setReadWrite(bool readWrite);
  virtual void updateBookmarksMenu(KPopupMenu *pop);
  virtual bool selectLine(int line);
  virtual bool canConfigureEditor() const { return true; }
  virtual void configureEditor();

public slots:
  void slotAddView();
  void slotRemoveCurrentView();
  void slotUndo();
  void slotRedo();
  void slotCut();
  void slotCopy();
  void slotPaste();
  void slotSearch();
  void slotReplace();
  void slotGotoLine();
  void slotToggleDosEOLOn();
  void slotToggleLineNumbersOn();
  void slotToggleBorderOn();
  void slotToggleWrapOn();
  virtual void slotStatusChanged();
  void slotSelectLine();
  void slotUnSelectLine();
  void slotGotoBookmark(int num);
  void slotUncomment();
  void slotComment();
  void slotFindNext();
  void slotFindPrevious();
  void slotPrintDialog();

  void slotGotFocus(Kate::View*);
  void slotHlChanged();

private:
  virtual bool load(const PURL &url);
  virtual bool save(const PURL &url);
  virtual void setModified(bool modified) { doc->setModified(modified); }
};

#endif
