/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MEMORY_DISPLAY_H
#define MEMORY_DISPLAY_H

#include <map>
#include <utility>

#include <qlabel.h>
#include <qptrlist.h>
class QVBoxLayout;

#include "HexValueEditor.h"
#include "memory.h"

//-----------------------------------------------------------------------------
class MemorySpaceEditor : public QWidget
{
  Q_OBJECT
public:
  MemorySpaceEditor(Memory &memory, MemorySpace space, bool withLabel,
                    QWidget *parent = 0, const char *name = 0);

  virtual void setReadWrite(bool readWrite) = 0;

public slots:
  virtual void updateDisplay() = 0;

signals:
  void modified();

protected:
  Memory      &_memory;
  MemorySpace  _space;
  QVBoxLayout *_top;
};

//-----------------------------------------------------------------------------
class MemoryDisplayerBase : public MemorySpaceEditor
{
  Q_OBJECT
public:
  MemoryDisplayerBase(QWidget *parent, Memory &memory, MemorySpace space,
                      uint lines, uint cols, uint offset = 0, int length = -1);

  virtual void setReadWrite(bool readWrite);

public slots:
  virtual void updateDisplay() { setBase(_firstAddress); }

private slots:
  void setFirstLine(int line) { setBase(line * _nbCols); }

private:
  uint _firstAddress, _nbLines, _nbCols;
  QPtrList<QLabel> adresses;
  QPtrList<HexValueEditor> editors;
  std::map<int, std::pair<int, QString> > specialLocs;

  void setBase(int address);
  void decodeSPDescriptor(const char *descriptor);
};

//-----------------------------------------------------------------------------
class MemoryDisplayer : public MemorySpaceEditor
{
  Q_OBJECT
public:
  MemoryDisplayer(QWidget *parent, Memory &memory, MemorySpace space,
                  uint lines, uint cols);

  virtual void setReadWrite(bool readWrite) { _base->setReadWrite(readWrite); }

public slots:
  virtual void updateDisplay() { _base->updateDisplay(); }

private:
  MemoryDisplayerBase *_base;
};

#endif
