/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "NewProject.h"

#include <klocale.h>
#include <kiconloader.h>
#include <kpushbutton.h>

#include "Project.h"
#include "Config.h"

//----------------------------------------------------------------------------
NewDialog::NewDialog(const QString &caption, QWidget *parent)
  : KDialogBase(parent, "new_dialog", true, caption, Ok|Cancel, Ok)
{
  QWidget *cw = new QWidget(this);
  setMainWidget(cw);
  _top = new QGridLayout(cw, 0, 0, 10, 10);

  _fLabel = new QLabel(cw);
  _top->addWidget(_fLabel, 0, 0);
  _filename = new QLineEdit(cw);
  connect(_filename, SIGNAL(textChanged(const QString &)), SLOT(changed()));
  _top->addMultiCellWidget(_filename, 0,0, 1,2);

  QLabel *label= new QLabel(i18n("Location:"), cw);
  _top->addWidget(label, 1, 0);
  _dir = new QLineEdit(cw);
  connect(_dir, SIGNAL(textChanged(const QString &)), SLOT(changed()));
  _top->addWidget(_dir, 1, 1);
  KIconLoader loader;
  QIconSet iconset = loader.loadIcon("fileopen", KIcon::Toolbar);
  KPushButton *button = new KPushButton(iconset, QString::null, cw);
  connect(button, SIGNAL(clicked()), SLOT(browse()));
  _top->addWidget(button, 1, 2);

  enableButtonOK(false);
}

void NewDialog::changed()
{
  enableButtonOK(!_filename->text().isEmpty() && !_dir->text().isEmpty());
}

void NewDialog::browse()
{
  PDirectory dir = PDirectory::getExisting(startDir(), this, QString::null);
  if ( dir.isEmpty() ) return;
  _dir->setText(dir.path());
}

PURL NewDialog::url() const
{
  return PURL(PDirectory(_dir->text()), _filename->text(), type());
}

//----------------------------------------------------------------------------
NewProjectDialog::NewProjectDialog(QWidget *parent)
  : NewDialog(i18n("Create New Project"), parent)
{
  _fLabel->setText("Project Name:");

  QLabel *label = new QLabel(i18n("Device:"), mainWidget());
  _top->addWidget(label, 2, 0);
  _deviceChooser = new DeviceChooser(Config::defaultDevice(), false, mainWidget());
  _top->addWidget(_deviceChooser, 2, 1);
}

//----------------------------------------------------------------------------
NewFileDialog::NewFileDialog(Project *project, QWidget *parent)
  : NewDialog(i18n("Create New File"), parent), _project(project)
{
  _fLabel->setText(i18n("File Name:"));

  _type = new QComboBox(mainWidget());
  for (uint i=0; i<PURL::Nb_FileTypes; i++) {
    if ( !(PURL::DATA[i].properties & PURL::Editable) ) continue;
    _type->insertItem(i18n(PURL::DATA[i].label));
  }
  connect(_type, SIGNAL(activated(int)), SLOT(typeChanged(int)));
  _top->addWidget(_type, 2, 0);

  _add = new QCheckBox(i18n("Add to project"), mainWidget());
  _add->setChecked(project);
  _top->addMultiCellWidget(_add, 2,2, 1,2);

  typeChanged(0);
  if (project) _dir->setText(project->directory().path());
}

PURL::FileType NewFileDialog::type() const
{
  for (int i=0; i<PURL::Nb_FileTypes; i++) {
    if ( !(PURL::DATA[i].properties & PURL::Editable) ) continue;
    if ( i==_type->currentItem() ) return PURL::FileType(i);
  }
  Q_ASSERT(false);
  return PURL::Nb_FileTypes;
}

void NewFileDialog::typeChanged(int)
{
  _add->setEnabled(_project && (PURL::DATA[type()].properties & PURL::Source));
}

PDirectory NewFileDialog::startDir() const
{
  if (_project) return _project->directory();
  return PDirectory::home();
}
