/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PIKMAIN_H
#define PIKMAIN_H

class QLabel;
class QProgressBar;

#include <kmainwindow.h>
class KProcess;

#include "EditorManager.h"
#include "ProjectWid.h"
#include "log.h"
#include "programmer.h"
class PrgWid;

//-----------------------------------------------------------------------------
class PikMain;

class CompileLog : public Log::Widget
{
  Q_OBJECT
public:
  CompileLog(QWidget *parent, PikMain *main);

public slots:
  void appendLine(const QString &line);
  virtual void clear();

private slots:
  void lineClicked(int line);

private:
  class Data {
  public:
    Data() {}
    Data(const PURL &u, uint l) : url(u), line(l) {}
    PURL url;
    uint line;
  };
  PikMain *_main;
  QMap<uint, Data> _map;
};

//-----------------------------------------------------------------------------
class PikMain : public KMainWindow
{
    Q_OBJECT
public:
    PikMain();
    virtual ~PikMain();

    KAction *action(const char *);
    ProjectWid *projectManager() { return project_wid; }
    EditorManager *editorManager() { return edimanager; }
    virtual bool queryClose();

private slots:
    void buildProject();
    void assembleFile();
    void program();
    void verify();
    void read();
    void erase();
    void blankCheck();
    void setTotalSteps(int total);
    void setProgress(int steps);

    void slotApplyToolbarSettings();
    void slotConfigureToolbar();
    void disAssemblePrg();
    void confEditor();
    void slotRunPikloops();
    void slotConfigCenter();

  void slotBookmarksManagement();
  void updateGUI();
  void initialLoading();
  void commandDone();
  void receivedStdout(KProcess *, char *buffer, int len);
  void receivedStderr(KProcess *, char *buffer, int len);
  void closeProject();
  void cleanProject();

signals:
    void SigNewFile() ;
    void SigOpenFile() ;
    void SigSaveFile() ;
    void SigSaveAsFile() ;
    void SigCloseFile() ;
    void SigQuitPrg() ;

    void SigCutSelection() ;
    void SigCopySelection() ;
    void SigPasteSelection() ;

    void SigAssemble() ;
    void SigProgram() ;

    void SigAbout();

private:
  KToolBar      *toolbar;
  EditorManager *edimanager;
  QTabWidget    *logTab;
  Log::Widget   *programLog;
  CompileLog    *compileLog;
  ProjectWid    *project_wid;
  QLabel        *_actionStatus, *_editorStatus, *_deviceStatus;
  QProgressBar  *_actionProgress;

  enum CommandType { AssembleStep = 0, Link, AssembleSingle, Disassemble, Nb_CommandTypes };
  static const char *COMMAND_LABELS[Nb_CommandTypes];
  class CommandData {
  public:
    CommandType type;
    PURL url;
    PURL::List todo;
    bool link;
  };
  CommandData  _commandData;
  KProcess    *_process;
  QString      _stdout, _stderr;
  PrgWid      *_progEditor;

  void saveSettings();
  void showExistingFile(const PURL &url, bool show, bool addToProject);
  void assembleProjectFile();
  void assembleStandAloneFile(const PURL &url);
  void linkProject();
  bool executeCommand(const PDirectory &dir, const QStringList &arguments);
  bool isExecutablePresent(const QStringList &args) const;
  bool hasGPUtils(const QString &executable);
  void prepareProgrammer();
  PrgWid *getHexEditor();
  PrgWid *createHexEditor(const QString &basename);
  void doAction(Programmer::Action action);
};

#endif
