/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "PrgConfigWidget.h"

#include <qlayout.h>
#include <qlabel.h>

#include <klocale.h>
#include <kiconloader.h>

#include "Config.h"
#include "programmer.h"

ProgConfigWidget::ProgConfigWidget()
{
  QGridLayout *grid = new QGridLayout(this);

  QLabel *label = new QLabel(i18n("Programmer in use:"), this);
  grid->addWidget(label, 0, 0);
  _combo = new QComboBox(this);
  connect(_combo, SIGNAL(activated(int)), SLOT(programmerChanged(int)));
  grid->addWidget(_combo, 0, 1);

  grid->setRowSpacing(1, 10);

  _stack = new QWidgetStack(this);
  grid->addMultiCellWidget(_stack, 2,2, 0,1);

  DeviceUser::Type dtype = Config::programmerType();
  uint k = 0;
  for (uint i=0; i<DeviceUser::Nb_Types; i++) {
    if ( !(DeviceUser::DATA[i].properties & DeviceUser::Programmer) ) continue;
    _combo->insertItem(i18n(DeviceUser::DATA[i].label));
    QWidget *w = Programmer::createConfigWidget(DeviceUser::Type(i), _stack);
    _stack->addWidget(w, i);
    if ( DeviceUser::Type(i)==dtype ) {
      _combo->setCurrentItem(k);
      _stack->raiseWidget(dtype);
    }
    k++;
  }
}

QString ProgConfigWidget::configName() const
{
  return i18n("Programmer Configuration");
}

QString ProgConfigWidget::title() const
{
  return i18n("Programmer");
}

QPixmap ProgConfigWidget::icon() const
{
  KIconLoader loader;
  return loader.loadIcon("piclab_config_prg", KIcon::Toolbar, KIcon::SizeMedium);
}

DeviceUser::Type ProgConfigWidget::type(int index) const
{
  int k = 0;
  for (uint i=0; i<DeviceUser::Nb_Types; i++) {
    if ( !(DeviceUser::DATA[i].properties & DeviceUser::Programmer) ) continue;
    if ( k==index ) return DeviceUser::Type(i);
    k++;
  }
  Q_ASSERT(false);
  return DeviceUser::Nb_Types;
}

void ProgConfigWidget::doApply()
{
  DeviceUser::Type t = type(_combo->currentItem());
  static_cast<ConfigWidget *>(_stack->widget(t))->doApply();
  Config::writeProgrammerType(t);
}

void ProgConfigWidget::programmerChanged(int i)
{
  _stack->raiseWidget(type(i));
}
