/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PRGWID_H
#define PRGWID_H

#include <qlabel.h>
class QVBoxLayout;

#include "editor.h"
#include "memory.h"
#include "programmer.h"
class DeviceChooser;
class MemorySpaceEditor;
class Project;

class PrgWid : public Editor
{
  Q_OBJECT
public:
  PrgWid(const QString &name, const QString &device, QWidget *parent);
  PrgWid(const PURL &url, const QString &device, QWidget *parent);
  PrgWid(const PURL &url, Project *project, QWidget *parent);
  virtual ~PrgWid();

  virtual int capabilities() const { return Common; }
  virtual PURL::FileType fileType() const { return PURL::Hex; }
  virtual uint value(ValueType type) const;
  virtual bool isModified() const { return _modified; }
  virtual void setModified(bool modified) { _modified = modified; }
  virtual void setReadWrite(bool readWrite);
  virtual bool isReadWrite() const { return _readWrite; }
  virtual bool selectLine(int) { return false; }
  virtual void updateBookmarksMenu(KPopupMenu *) {}
  virtual bool canConfigureEditor() const { return false; }
  virtual void configureEditor() {}
  QString device() const;
  void programmerChanged();
  bool doAction(Programmer::Action action);

public slots:
  virtual void slotStatusChanged() { emit statusChanged("  "); }

signals:
  void bufferModified();

private slots:
  void slotModified();
  void deviceChanged();

private:
  PURL _url;
  bool _modified, _supported, _readWrite;
  Memory _memory;
  Project *_project;
  QWidget *_widget;
  QVBoxLayout *_top, *_vbox;
  QLabel *_deviceLabel;
  DeviceChooser *_deviceChooser;
  QPtrList<MemorySpaceEditor> _editors;

  virtual void dropEvent(QDropEvent *e) { emit dropEventPass(e); }
  bool updateDisplay(const PURL &url, const QString &defaultDevice, QString &error);
  bool inProject() const;
  void reload();
  virtual bool load(const PURL &url);
  virtual bool save(const PURL &url);
  void init();
  void display();
  bool verifyDeviceType();
};

#endif
