/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROJECT_H
#define PROJECT_H

#include <kconfig.h>

#include "FNU.h"

/**A class to organize  project data
  *@author Alain Gibaud
  */

class Project
{
public:
  Project(const PURL &url);
  ~Project();

  PURL url() const { return _url; }
  PDirectory directory() const { return _url.directory(); }
  QString name() const { return _url.basename(); }
  PURL::List absoluteFiles() const;

  void setOutputFile(const PURL &url);
  PURL outputFile() const;
  void setVersion(const QString& id);
  QString version() const;
  QString description() const;
  void setDescription(const QString &);
  QString device() const;
  void setDevice(const QString &) ;
  QString hexFormat() const;
  void setHexFormat(const QString &) ;
  QString radix() const;
  void setRadix(const QString &) ;
  QString warnLevel() const;
  void setWarnLevel(const QString &) ;
  QString otherOptions() const;
  void setOtherOptions(const QString &) ;
  QString includeDir() const;
  void setIncludeDir(const QString &) ;
  bool linkDebug() const;
  void setLinkDebug(bool) ;
  QString libsDir() const;
  void setLibsDir(const QString &) ;
  QString otherLinkOptions() const;
  void setOtherLinkOptions(const QString &) ;

private:
  KConfig *config;
  PURL _url;

  QStringList relativeFiles() const;
  void saveFiles(const QStringList &urls);

public slots:
  void removeFile(const PURL &url); // take absolute filepath (but inside project dir)
  void addFile(const PURL &url); // take absolute filePath (but inside project dir)
  void restore();
};

#endif
