/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROJECTWID_H
#define PROJECTWID_H

#include <qwidget.h>
#include <qvbox.h>
#include <klistview.h>

#include "Project.h"
#include "EditorManager.h"

class QPopupMenu;
class PikMain;

class ListViewItem : public KListViewItem
{
public:
  enum FileGroup { Sources = 0, Includes, Objects, Generated, Nb_FileGroups };
  static FileGroup group(const PURL &url);

  ListViewItem(QListView *listview);
  ListViewItem(ListViewItem *item, FileGroup group);
  ListViewItem(ListViewItem *item, const PURL &url);
  void set(const PURL &url);
  PURL url() const { return _url; }
  PURL::FileType type() const { return _type; }
  FileGroup group() const { return _group; }

private:
  PURL _url;
  PURL::FileType _type;
  FileGroup _group;
  static const char *FILE_GROUP_LABELS[Nb_FileGroups];

  void init(PURL::FileType type);
};

class ProjectWid : public QVBox
{
  Q_OBJECT
public:
  ProjectWid(PikMain *mainwindow, QWidget *parent);

  void insertFile(const PURL &url);
  void openProjectFile(const PURL &url);
  Project *project() const { return _project; }
  void addFile(const PURL &url);
  void removeFile(const PURL &url);
  void select(const PURL &url);
  bool contains(const PURL &url) { return findItem(url); }

public slots:
  void slotProjectRButton(QListViewItem *, const QPoint &, int);
  void slotCloseProject();
  void slotInsertFile();
  void slotOpenProject();
  void slotInsertCurrentFile();
  void slotEditProject();
  void slotNewProject();
  void openProjectFile(const KURL &url) { openProjectFile(PURL(url)); }

signals:
  void guiChanged();

private:
  Project      *_project;
  PikMain      *_main;
  KListView    *_view;
  ListViewItem *_root;

  virtual void dragEnterEvent(QDragEnterEvent *);
  virtual void dropEvent(QDropEvent *ev);
  ListViewItem *findGroupItem(ListViewItem::FileGroup group);
  ListViewItem *findItem(const PURL &url) const;

private slots:
  void slotProjectElementClicked(QListViewItem *);
};

#endif
