/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef DEVICE_LIST_H
#define DEVICE_LIST_H

#include <qmap.h>
#include <qstringlist.h>
class QWidget;

#include "devices/devices.h"
class PURL;

class DeviceUser
{
public:
  DeviceUser() {}
  virtual ~DeviceUser() {}
  enum Type { GPAsm = 0, GPDAsm, DirectProgrammer, PicstartPlus,
              Promate2, ICD, ICD2Programmer, ICD2Debugger, Nb_Types,
              AllTypes = Nb_Types };
  enum Property { NoProperty = 0, Programmer = 1, Debugger = 2 };
  struct Data {
    const char *label;
    int properties;
  };
  static const Data DATA[Nb_Types];
  static int TYPE_MAPPING[Nb_Types];

  static Type defaultType(Property property);
  virtual Type type() const = 0;
};

class DeviceList
{
public:
  static QString defaultDevice(DeviceUser::Type type = DeviceUser::AllTypes);
  static QStringList names(DeviceUser::Type type = DeviceUser::AllTypes);
  static QString checkName(const QString &name);
  class Data {
  public:
    Data() : device(0), supported(DeviceUser::Nb_Types) { supported.fill(false); }
    const pic_device *device; // can return 0 for devices not in "devices" module
    QMemArray<bool> supported;
  };
  static const Data *data(const QString &name);
  static bool isSupported(const QString &name, DeviceUser::Type type);

  static QString guessDeviceFromFile(const PURL &url, bool &success, QWidget *widget);
  static QString toGPUtilsName(const QString &device);

private:
  static QMap<QString, Data> *_map;

  static QMap<QString, Data> &map();
  static void addDevice(const QString &name, DeviceUser::Type type);
  static void addGPAsmDevices();
  static void addGPDAsmDevices();
  static void addDirectProgDevices();
  static void addMicrochipDevices();
};

#endif
