/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2002-2003 Stephen Landamore <stephen@landamore.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef DEVICES_H
#define DEVICES_H

#ifdef __cplusplus
 extern "C" {
#endif

#ifndef NULL
#  define NULL 0
#endif

#define DEV_MAGIC 0 /* dummy */

enum Architecture { ARCH_16C5X = 0, ARCH_16X, ARCH_17X, ARCH_18X, ARCH_30X,
                    Nb_Architectures };

enum Supported { SUPPORT_DASM = 0x01, SUPPORT_PICSTART_PLUS = 0x02,
                 SUPPORT_PROMATE2 = 0x04, SUPPORT_ICD = 0x08,
                 SUPPORT_ICD2P = 0x10, SUPPORT_ICD2D = 0x20
};

enum MemoryTechnology { MEMTECH_EE = 0, MEMTECH_EPROM, MEMTECH_ROM,
                        MEMTECH_ROMLESS, MEMTECH_SS, Nb_MemoryTechnologies };
enum MemorySpace { MemoryCode = 0, MemoryCal, MemoryUserId, MemoryDeviceId,
                   MemoryConfig, MemoryEeprom, MemoryDbgvec, MemoryHwstack,
                   Nb_MemorySpaces };
struct pic_device_mem_spec {
    int present;
    int start, end;
    int width, inc, mask, init;
};

enum Icd2Family { ICD2_FAMILY_16F = 0, ICD2_FAMILY_16FA, ICD2_FAMILY_18C,
                  ICD2_FAMILY_18F, ICD2_FAMILY_18FEGG, ICD2_FAMILY_30F,
                  Nb_Icd2Families };

enum Icd2DebugExec {
    ICD2_DEBUGEXEC_DE12F629 = 0, ICD2_DEBUGEXEC_DE16F648A,
    ICD2_DEBUGEXEC_DE16F676, ICD2_DEBUGEXEC_DE16F684,
    ICD2_DEBUGEXEC_DE16F688, ICD2_DEBUGEXEC_DE16F716,
    ICD2_DEBUGEXEC_DE16F7X7, ICD2_DEBUGEXEC_DE16F819,
    ICD2_DEBUGEXEC_DE16F872, ICD2_DEBUGEXEC_DE16F874,
    ICD2_DEBUGEXEC_DE16F877, ICD2_DEBUGEXEC_DE16F88,
    ICD2_DEBUGEXEC_DE18CX01, ICD2_DEBUGEXEC_DE18F_BANK00,
    ICD2_DEBUGEXEC_DE18F_BANK01, ICD2_DEBUGEXEC_DE18F_BANK02,
    ICD2_DEBUGEXEC_DE18F_BANK05, ICD2_DEBUGEXEC_DE18F_BANK07,
    ICD2_DEBUGEXEC_DE18F_BANK12, ICD2_DEBUGEXEC_DE18F_BANK14,
    ICD2_DEBUGEXEC_DE30F_REVB, Nb_Icd2DebugExecs
};

enum Icd2DebugSet {
    ICD2_DEBUGSET_12F = 0, ICD2_DEBUGSET_16F, ICD2_DEBUGSET_16F648A,
    ICD2_DEBUGSET_16F684, ICD2_DEBUGSET_16F716, ICD2_DEBUGSET_16F7X7,
    ICD2_DEBUGSET_16F872, ICD2_DEBUGSET_18C, ICD2_DEBUGSET_18F,
    ICD2_DEBUGSET_30F, Nb_Icd2DebugSets
};

struct pic_config_bit {
    int mask;
    const char *mask_name;
    int val;
    const char *val_name;
};

struct pic_device {
    int magic;
    const char *name;
    int arch;
    int support;
    int memtech;
    const struct { float min, max, nominal; } vdd;
    const struct { float min, max, nominal; } vpp;
    int eeprom_hexfile_offset;
    const struct pic_device_mem_spec memory_code, memory_cal, memory_id,
        memory_config, memory_eeprom, memory_dbgvec, memory_hwstack;
    const struct { int firstrev, algorithm, overprogram, tries; } picstartplus;
    const struct { int firstrev, os, socket, part; } promate2;
    const struct { int firstrev, part; } icd;
    const struct { int family, efid, famid, debugexec, debugset; } icd2;
    const struct {
        int address;
        const char *name;
        const struct pic_config_bit *bits;
    } config[16];
};

extern const struct pic_device *PIC_DEVICES[];

#ifdef __cplusplus
 }
#endif

#endif
