/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
#include "devices.h"

static const struct pic_config_bit _dspic30f4012_fosc[] = {
    { .mask = 0xc000, .mask_name = "FCKSMEN", .val = 0x0000, .val_name = "Sw Enabled, Mon Enabled" },
    { .mask = 0xc000, .mask_name = "FCKSMEN", .val = 0x4000, .val_name = "Sw Enabled, Mon Disabled" },
    { .mask = 0xc000, .mask_name = "FCKSMEN", .val = 0x8000, .val_name = "Sw Disabled, Mon Disabled" },
    { .mask = 0x0300, .mask_name = "FOS", .val = 0x0000, .val_name = "Low Power 32kHz Osc <TMR1 Osc>" },
    { .mask = 0x0300, .mask_name = "FOS", .val = 0x0100, .val_name = "Internal Fast RC" },
    { .mask = 0x0300, .mask_name = "FOS", .val = 0x0200, .val_name = "Internal Low Power RC" },
    { .mask = 0x0300, .mask_name = "FOS", .val = 0x0300, .val_name = "Primary Oscillator" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0000, .val_name = "XTL" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0002, .val_name = "HS" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0004, .val_name = "XT" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0005, .val_name = "XT w/PLL 4x" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0006, .val_name = "XT w/PLL 8x" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0007, .val_name = "XT w/PLL 16x" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0008, .val_name = "ERCIO" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x0009, .val_name = "ERC" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x000b, .val_name = "EC" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x000c, .val_name = "ECIO" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x000d, .val_name = "EC w/ PLL 4x" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x000e, .val_name = "EC w/ PLL 8x" },
    { .mask = 0x000f, .mask_name = "FPR", .val = 0x000f, .val_name = "EC w/ PLL 16x" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _dspic30f4012_fwdt[] = {
    { .mask = 0x8000, .mask_name = "WDT", .val = 0x0000, .val_name = "Disabled" },
    { .mask = 0x8000, .mask_name = "WDT", .val = 0x8000, .val_name = "Enabled" },
    { .mask = 0x0030, .mask_name = "FWPSA", .val = 0x00, .val_name = "1:1" },
    { .mask = 0x0030, .mask_name = "FWPSA", .val = 0x10, .val_name = "1:8" },
    { .mask = 0x0030, .mask_name = "FWPSA", .val = 0x20, .val_name = "1:64" },
    { .mask = 0x0030, .mask_name = "FWPSA", .val = 0x30, .val_name = "1:512" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x0, .val_name = "1:1" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x1, .val_name = "1:2" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x2, .val_name = "1:3" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x3, .val_name = "1:4" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x4, .val_name = "1:5" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x5, .val_name = "1:6" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x6, .val_name = "1:7" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x7, .val_name = "1:8" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x8, .val_name = "1:9" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0x9, .val_name = "1:10" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0xa, .val_name = "1:11" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0xb, .val_name = "1:12" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0xc, .val_name = "1:13" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0xd, .val_name = "1:14" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0xe, .val_name = "1:15" },
    { .mask = 0x000f, .mask_name = "FWPSB", .val = 0xf, .val_name = "1:16" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _dspic30f4012_fborpor[] = {
    { .mask = 0x8000, .mask_name = "MCLRE", .val = 0x0000, .val_name = "Disabled" },
    { .mask = 0x8000, .mask_name = "MCLRE", .val = 0x8000, .val_name = "Enabled" },
    { .mask = 0x0400, .mask_name = "PWMPIN", .val = 0x000, .val_name = "Control with HPOL/LPOL bits" },
    { .mask = 0x0400, .mask_name = "PWMPIN", .val = 0x400, .val_name = "Control with PORT/TRIS regs" },
    { .mask = 0x0200, .mask_name = "HPOL", .val = 0x0000, .val_name = "Active Low" },
    { .mask = 0x0200, .mask_name = "HPOL", .val = 0x0200, .val_name = "Active High" },
    { .mask = 0x0100, .mask_name = "LPOL", .val = 0x0000, .val_name = "Active Low" },
    { .mask = 0x0100, .mask_name = "LPOL", .val = 0x0100, .val_name = "Active High" },
    { .mask = 0x0080, .mask_name = "BOREN", .val = 0x00, .val_name = "Disabled" },
    { .mask = 0x0080, .mask_name = "BOREN", .val = 0x80, .val_name = "Enabled" },
    { .mask = 0x0030, .mask_name = "BODENV", .val = 0x00, .val_name = "4.5V" },
    { .mask = 0x0030, .mask_name = "BODENV", .val = 0x10, .val_name = "4.2V" },
    { .mask = 0x0030, .mask_name = "BODENV", .val = 0x20, .val_name = "2.7V" },
    { .mask = 0x0030, .mask_name = "BODENV", .val = 0x30, .val_name = "2.0V" },
    { .mask = 0x0003, .mask_name = "FPWRT", .val = 0x0, .val_name = "Timer Disabled" },
    { .mask = 0x0003, .mask_name = "FPWRT", .val = 0x1, .val_name = "4ms" },
    { .mask = 0x0003, .mask_name = "FPWRT", .val = 0x2, .val_name = "16ms" },
    { .mask = 0x0003, .mask_name = "FPWRT", .val = 0x3, .val_name = "64ms" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _dspic30f4012_reserved1[] = {
    { .mask = 0x3000, .mask_name = "RES1", .val = 0x3000, .val_name = "Reserved" },
    { .mask = 0x300, .mask_name = "RES2", .val = 0x300, .val_name = "Reserved" },
    { .mask = 0xf, .mask_name = "RES3", .val = 0xf, .val_name = "Reserved" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _dspic30f4012_reserved2[] = {
    { .mask = 0x3000, .mask_name = "RES1", .val = 0x3000, .val_name = "Reserved" },
    { .mask = 0x300, .mask_name = "RES2", .val = 0x300, .val_name = "Reserved" },
    { .mask = 0xf, .mask_name = "RES3", .val = 0xf, .val_name = "Reserved" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _dspic30f4012_fgs[] = {
    { .mask = 0x0004, .mask_name = "RES1", .val = 0x0, .val_name = "Reserved" },
    { .mask = 0x0002, .mask_name = "GCP", .val = 0x0000, .val_name = "Enabled" },
    { .mask = 0x0002, .mask_name = "GCP", .val = 0x0002, .val_name = "Disabled" },
    { .mask = 0x0001, .mask_name = "GWRP", .val = 0x0000, .val_name = "Enabled" },
    { .mask = 0x0001, .mask_name = "GWRP", .val = 0x0001, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _dspic30f4012_icd[] = {
    { .mask = 0x8000, .mask_name = "BACKBUG", .val = 0x0000, .val_name = "Enabled" },
    { .mask = 0x8000, .mask_name = "BACKBUG", .val = 0x8000, .val_name = "Disabled" },
    { .mask = 0x4000, .mask_name = "COE", .val = 0x0000, .val_name = "Reset Into Clip On Emulation Mode" },
    { .mask = 0x4000, .mask_name = "COE", .val = 0x4000, .val_name = "Reset Into Operational Mode" },
    { .mask = 0x0003, .mask_name = "ICS", .val = 0x0, .val_name = "Use EMUC3 and EMUD3" },
    { .mask = 0x0003, .mask_name = "ICS", .val = 0x1, .val_name = "Use EMUC2 and EMUD2" },
    { .mask = 0x0003, .mask_name = "ICS", .val = 0x2, .val_name = "Use EMUC1 and EMUD1" },
    { .mask = 0x0003, .mask_name = "ICS", .val = 0x3, .val_name = "Use PGC/EMUC and PGD/EMUD" },
    { .mask_name = NULL }
};

static const struct pic_device _dspic30f4012 = {
    .magic = DEV_MAGIC,
    .name = "dsPIC30F4012",
    .arch = ARCH_30X,
    .support = SUPPORT_DASM | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.500, .max = 13.500, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0,
    .memory_code = {
        .present = 1,
        .start = 0x0,
        .end  = 0x7fff,
        .width = 4,
        .inc = 2,
        .mask = 0x00ffffff,
        .init = 0x00ffffff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x8005c0,
        .end  = 0x8005ff,
        .width = 4,
        .inc = 2,
        .mask = 0x00ffffff,
        .init = 0x00ffffff
    },
    .memory_config = {
        .present = 1,
        .start = 0xf80000,
        .end  = 0xf8000d,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_eeprom = {
        .present = 1,
        .start = 0x7ffc00,
        .end  = 0x7fffff,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x800000,
        .end  = 0x800007,
        .width = 4,
        .inc = 2,
        .mask = 0xff7fff,
        .init = 0xffffff
    },
    .promate2 = {
        .firstrev = 0x060000,
        .os       = 0x13,
        .socket   = 0x50,
        .part     = 0x0d
    },
    .icd2 = {
        .family = ICD2_FAMILY_30F,
        .efid   = 0x03,
        .famid  = 0xbb,
        .debugexec = ICD2_DEBUGEXEC_DE30F_REVB,
        .debugset = ICD2_DEBUGSET_30F
    },
    .config = {
        { .address = 0xf80000, .name = "FOSC", .bits = _dspic30f4012_fosc },
        { .address = 0xf80002, .name = "FWDT", .bits = _dspic30f4012_fwdt },
        { .address = 0xf80004, .name = "FBORPOR", .bits = _dspic30f4012_fborpor },
        { .address = 0xf80006, .name = "RESERVED1", .bits = _dspic30f4012_reserved1 },
        { .address = 0xf80008, .name = "RESERVED2", .bits = _dspic30f4012_reserved2 },
        { .address = 0xf8000a, .name = "FGS", .bits = _dspic30f4012_fgs },
        { .address = 0xf8000c, .name = "ICD", .bits = _dspic30f4012_icd },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
