/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
#include "devices.h"

static const struct pic_config_bit _pic12f508_config[] = {
    { .mask = 0x10, .mask_name = "MCLRE", .val = 0x0, .val_name = "Internal" },
    { .mask = 0x10, .mask_name = "MCLRE", .val = 0x10, .val_name = "External" },
    { .mask = 0x8, .mask_name = "CP", .val = 0x0, .val_name = "On" },
    { .mask = 0x8, .mask_name = "CP", .val = 0x8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x2, .val_name = "INTOSC" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x3, .val_name = "EXTRC" },
    { .mask_name = NULL }
};

static const struct pic_device _pic12f508 = {
    .magic = DEV_MAGIC,
    .name = "PIC12F508",
    .arch = ARCH_16C5X,
    .support = SUPPORT_DASM | SUPPORT_PROMATE2 | SUPPORT_ICD2P,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x1ff,
        .width = 2,
        .inc = 1,
        .mask = 0x0fff,
        .init = 0x0fff
    },
    .memory_cal = {
        .present = 1,
        .start = 0x1ff,
        .end  = 0x1ff,
        .width = 2,
        .inc = 1,
        .mask = 0x0fff,
        .init = 0x0fff
    },
    .memory_id = {
        .present = 1,
        .start = 0x200,
        .end  = 0x203,
        .width = 2,
        .inc = 1,
        .mask = 0x000f,
        .init = 0x0fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x3ff,
        .end  = 0x3ff,
        .width = 2,
        .inc = 1,
        .mask = 0x0fff,
        .init = 0x0fff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .promate2 = {
        .firstrev = 0x062026,
        .os       = 0x10,
        .socket   = 0x60,
        .part     = 0x1e
    },
    .icd2 = {
        .family = ICD2_FAMILY_16F,
        .efid   = 0x07,
        .famid  = 0x11
    },
    .config = {
        { .address = 0x3ff, .name = "CONFIG", .bits = _pic12f508_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
