/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
#include "devices.h"

static const struct pic_config_bit _pic14000_config[] = {
    { .mask = 0x3fb0, .mask_name = "CP", .val = 0x0, .val_name = "On" },
    { .mask = 0x3fb0, .mask_name = "CP", .val = 0x1e30, .val_name = "Off OTP" },
    { .mask = 0x3fb0, .mask_name = "CP", .val = 0x3fb0, .val_name = "Off" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x0, .val_name = "On" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask = 0x1, .mask_name = "OSC", .val = 0x0, .val_name = "HS" },
    { .mask = 0x1, .mask_name = "OSC", .val = 0x1, .val_name = "Internal RC" },
    { .mask_name = NULL }
};

static const struct pic_device _pic14000 = {
    .magic = DEV_MAGIC,
    .name = "PIC14000",
    .arch = ARCH_16X,
    .support = SUPPORT_DASM,
    .memtech = MEMTECH_EPROM,
    .vdd = { .min = 2.750, .max = 6.000, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0x2100,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0xfff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_cal = {
        .present = 1,
        .start = 0xfc0,
        .end  = 0xfff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_id = {
        .present = 1,
        .start = 0x2000,
        .end  = 0x2003,
        .width = 2,
        .inc = 1,
        .mask = 0x007f,
        .init = 0x3fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x2007,
        .end  = 0x2007,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .config = {
        { .address = 0x2007, .name = "CONFIG", .bits = _pic14000_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
