/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic16c765_config[] = {
    { .mask = 0x3f30, .mask_name = "CP", .val = 0x0, .val_name = "All" },
    { .mask = 0x3f30, .mask_name = "CP", .val = 0x1510, .val_name = "800:1FFF" },
    { .mask = 0x3f30, .mask_name = "CP", .val = 0x2a20, .val_name = "1000:1FFF" },
    { .mask = 0x3f30, .mask_name = "CP", .val = 0x3f30, .val_name = "Off" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x0, .val_name = "On" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x0, .val_name = "HS" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x1, .val_name = "EC" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x2, .val_name = "H4" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x3, .val_name = "E4" },
    { .mask_name = NULL }
};

static const struct pic_device _pic16c765 = {
    .magic = DEV_MAGIC,
    .name = "PIC16C765",
    .arch = ARCH_16X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2,
    .memtech = MEMTECH_EPROM,
    .vdd = { .min = 4.375, .max = 5.250, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0x2100,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x1fff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x2000,
        .end  = 0x2003,
        .width = 2,
        .inc = 1,
        .mask = 0x007f,
        .init = 0x3fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x2007,
        .end  = 0x2007,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .picstartplus = {
        .firstrev    = 0x020100,
        .algorithm   = 4,
        .overprogram = 3,
        .tries       = 25
    },
    .promate2 = {
        .firstrev = 0x051010,
        .os       = 0x10,
        .socket   = 0xb0,
        .part     = 0x30
    },
    .config = {
        { .address = 0x2007, .name = "CONFIG", .bits = _pic16c765_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
