/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic16cr58b_config[] = {
    { .mask = 0xff8, .mask_name = "CP", .val = 0x0, .val_name = "On" },
    { .mask = 0xff8, .mask_name = "CP", .val = 0xff8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x3, .val_name = "RC" },
    { .mask_name = NULL }
};

static const struct pic_device _pic16cr58b = {
    .magic = DEV_MAGIC,
    .name = "PIC16CR58B",
    .arch = ARCH_16C5X,
    .support = SUPPORT_DASM | SUPPORT_PROMATE2,
    .memtech = MEMTECH_ROM,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.500, .max = 13.500, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x7ff,
        .width = 2,
        .inc = 1,
        .mask = 0x0fff,
        .init = 0x0fff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x800,
        .end  = 0x803,
        .width = 2,
        .inc = 1,
        .mask = 0x000f,
        .init = 0x0fff
    },
    .memory_config = {
        .present = 1,
        .start = 0xfff,
        .end  = 0xfff,
        .width = 2,
        .inc = 1,
        .mask = 0x0fff,
        .init = 0x0fff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .promate2 = {
        .firstrev = 0x040008,
        .os       = 0x10,
        .socket   = 0xe0,
        .part     = 0x19
    },
    .config = {
        { .address = 0xfff, .name = "CONFIG", .bits = _pic16cr58b_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
