/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic16f630_config[] = {
    { .mask = 0x3000, .mask_name = "BG", .val = 0x0000, .val_name = "Lowest Bandgap Voltage" },
    { .mask = 0x3000, .mask_name = "BG", .val = 0x3000, .val_name = "Highest Bandgap Voltage" },
    { .mask = 0x100, .mask_name = "CPD", .val = 0x0, .val_name = "On" },
    { .mask = 0x100, .mask_name = "CPD", .val = 0x100, .val_name = "Off" },
    { .mask = 0x80, .mask_name = "CP", .val = 0x00, .val_name = "On" },
    { .mask = 0x80, .mask_name = "CP", .val = 0x80, .val_name = "Off" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x0, .val_name = "Off" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x40, .val_name = "On" },
    { .mask = 0x20, .mask_name = "MCLRE", .val = 0x00, .val_name = "Internal" },
    { .mask = 0x20, .mask_name = "MCLRE", .val = 0x20, .val_name = "External" },
    { .mask = 0x10, .mask_name = "PUT", .val = 0x0, .val_name = "On" },
    { .mask = 0x10, .mask_name = "PUT", .val = 0x10, .val_name = "Off" },
    { .mask = 0x8, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x8, .mask_name = "WDT", .val = 0x8, .val_name = "On" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x3, .val_name = "EC" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x4, .val_name = "Internal RC No Clock" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x5, .val_name = "Internal RC Clockout" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x6, .val_name = "External RC No Clock" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x7, .val_name = "External RC Clockout" },
    { .mask_name = NULL }
};

static const struct pic_device _pic16f630 = {
    .magic = DEV_MAGIC,
    .name = "PIC16F630",
    .arch = ARCH_16X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0x2100,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x3ff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_cal = {
        .present = 1,
        .start = 0x3ff,
        .end  = 0x3ff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_id = {
        .present = 1,
        .start = 0x2000,
        .end  = 0x2003,
        .width = 2,
        .inc = 1,
        .mask = 0x007f,
        .init = 0x3fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x2007,
        .end  = 0x2007,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_eeprom = {
        .present = 1,
        .start = 0x0,
        .end  = 0x7f,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x2004,
        .end  = 0x2004,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .picstartplus = {
        .firstrev    = 0x030006,
        .algorithm   = 15,
        .overprogram = 0,
        .tries       = 1
    },
    .promate2 = {
        .firstrev = 0x058005,
        .os       = 0x10,
        .socket   = 0x60,
        .part     = 0x16
    },
    .icd2 = {
        .family = ICD2_FAMILY_16F,
        .efid   = 0x01,
        .famid  = 0x05,
        .debugexec = ICD2_DEBUGEXEC_DE16F676,
        .debugset = ICD2_DEBUGSET_12F
    },
    .config = {
        { .address = 0x2007, .name = "CONFIG", .bits = _pic16f630_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
