/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic16f777_config[] = {
    { .mask = 0x2000, .mask_name = "CP", .val = 0x0, .val_name = "On" },
    { .mask = 0x2000, .mask_name = "CP", .val = 0x2000, .val_name = "Off" },
    { .mask = 0x1000, .mask_name = "CCP2MUX", .val = 0x0, .val_name = "RB3" },
    { .mask = 0x1000, .mask_name = "CCP2MUX", .val = 0x1000, .val_name = "RC1" },
    { .mask = 0x800, .mask_name = "BACKBUG", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x800, .mask_name = "BACKBUG", .val = 0x800, .val_name = "Disabled" },
    { .mask = 0x180, .mask_name = "BODENV", .val = 0x000, .val_name = "Set to 4.5V" },
    { .mask = 0x180, .mask_name = "BODENV", .val = 0x080, .val_name = "Set to 4.2V" },
    { .mask = 0x180, .mask_name = "BODENV", .val = 0x100, .val_name = "Set to 2.7V" },
    { .mask = 0x180, .mask_name = "BODENV", .val = 0x180, .val_name = "Set to 2.0V" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x0, .val_name = "Disable" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x40, .val_name = "Enable" },
    { .mask = 0x20, .mask_name = "MCLRE", .val = 0x0, .val_name = "RE3" },
    { .mask = 0x20, .mask_name = "MCLRE", .val = 0x20, .val_name = "MCLR" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x00, .val_name = "LP" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x01, .val_name = "XT" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x02, .val_name = "HS" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x03, .val_name = "EXTCLK as Port IO" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x10, .val_name = "INTRC as Port I/O" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x11, .val_name = "INTRC as Clock Out" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x12, .val_name = "EXTRC as Port I/O" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x13, .val_name = "EXTRC as Clock Out" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x0, .val_name = "On" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic16f777_config2[] = {
    { .mask = 0x40, .mask_name = "BORSEN", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x40, .mask_name = "BORSEN", .val = 0x40, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "IESO", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x2, .mask_name = "IESO", .val = 0x2, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "FCMEN", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "FCMEN", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic16f777_config3[] = {
    { .mask_name = NULL }
};

static const struct pic_device _pic16f777 = {
    .magic = DEV_MAGIC,
    .name = "PIC16F777",
    .arch = ARCH_16X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.500, .max = 13.500, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0x2100,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x1fff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x2000,
        .end  = 0x2003,
        .width = 2,
        .inc = 1,
        .mask = 0x007f,
        .init = 0x3fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x2007,
        .end  = 0x2008,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x2004,
        .end  = 0x2004,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .picstartplus = {
        .firstrev    = 0x041000,
        .algorithm   = 23,
        .overprogram = 3,
        .tries       = 1
    },
    .promate2 = {
        .firstrev = 0x060005,
        .os       = 0x10,
        .socket   = 0xb0,
        .part     = 0x44
    },
    .icd2 = {
        .family = ICD2_FAMILY_16F,
        .efid   = 0x01,
        .famid  = 0x0a,
        .debugexec = ICD2_DEBUGEXEC_DE16F7X7,
        .debugset = ICD2_DEBUGSET_16F7X7
    },
    .config = {
        { .address = 0x2007, .name = "CONFIG", .bits = _pic16f777_config },
        { .address = 0x2008, .name = "CONFIG2", .bits = _pic16f777_config2 },
        { .address = 0x2009, .name = "CONFIG3", .bits = _pic16f777_config3 },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
