/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic16f819_config[] = {
    { .mask = 0x2000, .mask_name = "CP", .val = 0x0, .val_name = "Enabled - All protected" },
    { .mask = 0x2000, .mask_name = "CP", .val = 0x2000, .val_name = "Disabled" },
    { .mask = 0x1000, .mask_name = "CCP1MUX", .val = 0x0, .val_name = "RB3" },
    { .mask = 0x1000, .mask_name = "CCP1MUX", .val = 0x1000, .val_name = "RB2" },
    { .mask = 0x0800, .mask_name = "BACKBUG", .val = 0x0000, .val_name = "Enabled" },
    { .mask = 0x0800, .mask_name = "BACKBUG", .val = 0x0800, .val_name = "Disabled" },
    { .mask = 0x600, .mask_name = "WRT_ENABLE", .val = 0x000, .val_name = "0x0000-0x5FF Write Protected, 0x600-0x7FF may be modified by EECON" },
    { .mask = 0x600, .mask_name = "WRT_ENABLE", .val = 0x200, .val_name = "0x0000-0x3FF Write Protected, 0x400-0x7FF may be modified by EECON" },
    { .mask = 0x600, .mask_name = "WRT_ENABLE", .val = 0x400, .val_name = "0x0000-0x1FF Write Protected, 0x200-0x7FF may be modified by EECON" },
    { .mask = 0x600, .mask_name = "WRT_ENABLE", .val = 0x600, .val_name = "Write Protection Disabled" },
    { .mask = 0x100, .mask_name = "CPD", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x100, .mask_name = "CPD", .val = 0x100, .val_name = "Disabled" },
    { .mask = 0x80, .mask_name = "LVP", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x80, .mask_name = "LVP", .val = 0x80, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x0, .val_name = "Off" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x40, .val_name = "On" },
    { .mask = 0x20, .mask_name = "MCLRE", .val = 0x0, .val_name = "RA5 is digital I/O, MCLR tied to VDD" },
    { .mask = 0x20, .mask_name = "MCLRE", .val = 0x20, .val_name = "RA5 is MCLR" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x3, .val_name = "EXTCLK-RA6 is Port I/O" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x10, .val_name = "INTRC-RA6 is Port I/O" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x11, .val_name = "INTRC-RA6 is CLKOUT" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x12, .val_name = "EXTRC-RA6 is Port I/O" },
    { .mask = 0x13, .mask_name = "OSC", .val = 0x13, .val_name = "EXTRC-RA6 is CLKOUT" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x0, .val_name = "On" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask_name = NULL }
};

static const struct pic_device _pic16f819 = {
    .magic = DEV_MAGIC,
    .name = "PIC16F819",
    .arch = ARCH_16X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.500, .max = 13.500, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0x2100,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x7ff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x2000,
        .end  = 0x2003,
        .width = 2,
        .inc = 1,
        .mask = 0x007f,
        .init = 0x3fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x2007,
        .end  = 0x2007,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_eeprom = {
        .present = 1,
        .start = 0x0,
        .end  = 0xff,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x2004,
        .end  = 0x2004,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .picstartplus = {
        .firstrev    = 0x030007,
        .algorithm   = 16,
        .overprogram = 0,
        .tries       = 1
    },
    .promate2 = {
        .firstrev = 0x058021,
        .os       = 0x10,
        .socket   = 0xc0,
        .part     = 0x2a
    },
    .icd2 = {
        .family = ICD2_FAMILY_16FA,
        .efid   = 0x01,
        .famid  = 0x06,
        .debugexec = ICD2_DEBUGEXEC_DE16F819,
        .debugset = ICD2_DEBUGSET_16F
    },
    .config = {
        { .address = 0x2007, .name = "CONFIG", .bits = _pic16f819_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
