/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic16f871_config[] = {
    { .mask = 0x3030, .mask_name = "CP", .val = 0x0, .val_name = "On" },
    { .mask = 0x3030, .mask_name = "CP", .val = 0x3030, .val_name = "Off" },
    { .mask = 0x800, .mask_name = "BACKBUG", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x800, .mask_name = "BACKBUG", .val = 0x800, .val_name = "Disabled" },
    { .mask = 0x200, .mask_name = "WRT_ENABLE", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x200, .mask_name = "WRT_ENABLE", .val = 0x200, .val_name = "Enabled" },
    { .mask = 0x100, .mask_name = "CPD", .val = 0x0, .val_name = "On" },
    { .mask = 0x100, .mask_name = "CPD", .val = 0x100, .val_name = "Off" },
    { .mask = 0x80, .mask_name = "LVP", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x80, .mask_name = "LVP", .val = 0x80, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x0, .val_name = "Off" },
    { .mask = 0x40, .mask_name = "BODEN", .val = 0x40, .val_name = "On" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x0, .val_name = "On" },
    { .mask = 0x8, .mask_name = "PUT", .val = 0x8, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4, .mask_name = "WDT", .val = 0x4, .val_name = "On" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x3, .val_name = "RC" },
    { .mask_name = NULL }
};

static const struct pic_device _pic16f871 = {
    .magic = DEV_MAGIC,
    .name = "PIC16F871",
    .arch = ARCH_16X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2 | SUPPORT_ICD | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.500, .max = 13.500, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0x2100,
    .memory_code = {
        .present = 1,
        .start = 0x00,
        .end  = 0x7ff,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x2000,
        .end  = 0x2003,
        .width = 2,
        .inc = 1,
        .mask = 0x007f,
        .init = 0x3fff
    },
    .memory_config = {
        .present = 1,
        .start = 0x2007,
        .end  = 0x2007,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .memory_eeprom = {
        .present = 1,
        .start = 0x0,
        .end  = 0x3f,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x2004,
        .end  = 0x2004,
        .width = 2,
        .inc = 1,
        .mask = 0x3fff,
        .init = 0x3fff
    },
    .picstartplus = {
        .firstrev    = 0x020100,
        .algorithm   = 4,
        .overprogram = 0,
        .tries       = 1
    },
    .promate2 = {
        .firstrev = 0x051010,
        .os       = 0x10,
        .socket   = 0xb0,
        .part     = 0x2c
    },
    .icd = {
        .firstrev = 0x023100,
        .part     = 2
    },
    .icd2 = {
        .family = ICD2_FAMILY_16F,
        .efid   = 0x01,
        .famid  = 0x03,
        .debugexec = ICD2_DEBUGEXEC_DE16F872,
        .debugset = ICD2_DEBUGSET_16F872
    },
    .config = {
        { .address = 0x2007, .name = "CONFIG", .bits = _pic16f871_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
