/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic17c752_config[] = {
    { .mask = 0x8050, .mask_name = "PMODE", .val = 0x0, .val_name = "Prot. Microcontroller" },
    { .mask = 0x8050, .mask_name = "PMODE", .val = 0x8010, .val_name = "Ext. Microcontroller" },
    { .mask = 0x8050, .mask_name = "PMODE", .val = 0x8040, .val_name = "Microcontroller" },
    { .mask = 0x8050, .mask_name = "PMODE", .val = 0x8050, .val_name = "Microprocessor" },
    { .mask = 0x4000, .mask_name = "BODEN", .val = 0x0, .val_name = "Off" },
    { .mask = 0x4000, .mask_name = "BODEN", .val = 0x4000, .val_name = "On" },
    { .mask = 0xc, .mask_name = "WDT", .val = 0x0, .val_name = "TMR" },
    { .mask = 0xc, .mask_name = "WDT", .val = 0x4, .val_name = "x128" },
    { .mask = 0xc, .mask_name = "WDT", .val = 0x8, .val_name = "x512" },
    { .mask = 0xc, .mask_name = "WDT", .val = 0xc, .val_name = "x1" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x0, .val_name = "LF" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x1, .val_name = "RC" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x2, .val_name = "XT" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x3, .val_name = "EC" },
    { .mask_name = NULL }
};

static const struct pic_device _pic17c752 = {
    .magic = DEV_MAGIC,
    .name = "PIC17C752",
    .arch = ARCH_17X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2,
    .memtech = MEMTECH_EPROM,
    .vdd = { .min = 3.000, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0,
    .memory_code = {
        .present = 1,
        .start = 0x0,
        .end  = 0x1fff,
        .width = 2,
        .inc = 1,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_cal = { .present = 0 },
    .memory_id = { .present = 0 },
    .memory_config = {
        .present = 1,
        .start = 0xfe00,
        .end  = 0xfe00,
        .width = 2,
        .inc = 1,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .picstartplus = {
        .firstrev    = 0x020100,
        .algorithm   = 7,
        .overprogram = 3,
        .tries       = 25
    },
    .promate2 = {
        .firstrev = 0x042235,
        .os       = 0x12,
        .socket   = 0xd0,
        .part     = 0x07
    },
    .config = {
        { .address = 0xfe00, .name = "CONFIG", .bits = _pic17c752_config },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
