/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic18c601_config1h[] = {
    { .mask = 0x3, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x1, .val_name = "EC" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0x3, .mask_name = "OSC", .val = 0x3, .val_name = "RC" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c601_config2l[] = {
    { .mask = 0x40, .mask_name = "BUS", .val = 0x0, .val_name = "8 Bit" },
    { .mask = 0x40, .mask_name = "BUS", .val = 0x40, .val_name = "16 Bit" },
    { .mask = 0x1, .mask_name = "PUT", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "PUT", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c601_config2h[] = {
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x0, .val_name = "1:1" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x2, .val_name = "1:2" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x4, .val_name = "1:4" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x6, .val_name = "1:8" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x8, .val_name = "1:16" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0xa, .val_name = "1:32" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0xc, .val_name = "1:64" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0xe, .val_name = "1:128" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c601_config4l[] = {
    { .mask = 0x80, .mask_name = "BACKBUG", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "BACKBUG", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_device _pic18c601 = {
    .magic = DEV_MAGIC,
    .name = "PIC18C601",
    .arch = ARCH_18X,
    .support = SUPPORT_DASM | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_ROMLESS,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0xf00000,
    .memory_code = { .present = 0 },
    .memory_cal = { .present = 0 },
    .memory_id = { .present = 0 },
    .memory_config = {
        .present = 1,
        .start = 0x300000,
        .end  = 0x300007,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .promate2 = {
        .firstrev = 0x054001,
        .os       = 0x11,
        .socket   = 0xd0,
        .part     = 0x0d
    },
    .icd2 = {
        .family = ICD2_FAMILY_18C,
        .efid   = 0x06,
        .famid  = 0x81,
        .debugexec = ICD2_DEBUGEXEC_DE18CX01,
        .debugset = ICD2_DEBUGSET_18C
    },
    .config = {
        { .address = 0x300001, .name = "CONFIG1H", .bits = _pic18c601_config1h },
        { .address = 0x300002, .name = "CONFIG2L", .bits = _pic18c601_config2l },
        { .address = 0x300003, .name = "CONFIG2H", .bits = _pic18c601_config2h },
        { .address = 0x300006, .name = "CONFIG4L", .bits = _pic18c601_config4l },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
