/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic18c658_config1l[] = {
    { .mask = 0xff, .mask_name = "CP", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0xff, .mask_name = "CP", .val = 0xff, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c658_config1h[] = {
    { .mask = 0x80, .mask_name = "RES1", .val = 0x80, .val_name = "Reserved" },
    { .mask = 0x40, .mask_name = "RES2", .val = 0x40, .val_name = "Reserved" },
    { .mask = 0x20, .mask_name = "OSCS", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x20, .mask_name = "OSCS", .val = 0x20, .val_name = "Disabled" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x3, .val_name = "RC" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x4, .val_name = "EC-OSC2 as Clock Out" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x5, .val_name = "EC-OSC2 as RA6" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x6, .val_name = "HS-PLL Enabled" },
    { .mask = 0x7, .mask_name = "OSC", .val = 0x7, .val_name = "RC-OSC2 as RA6" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c658_config2l[] = {
    { .mask = 0xc, .mask_name = "BODENV", .val = 0x0, .val_name = "4.5V" },
    { .mask = 0xc, .mask_name = "BODENV", .val = 0x4, .val_name = "4.2V" },
    { .mask = 0xc, .mask_name = "BODENV", .val = 0x8, .val_name = "2.7V" },
    { .mask = 0xc, .mask_name = "BODENV", .val = 0xc, .val_name = "2.5V" },
    { .mask = 0x2, .mask_name = "BODEN", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x2, .mask_name = "BODEN", .val = 0x2, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "PUT", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "PUT", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c658_config2h[] = {
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x0, .val_name = "1:1" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x2, .val_name = "1:2" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x4, .val_name = "1:4" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x6, .val_name = "1:8" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0x8, .val_name = "1:16" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0xa, .val_name = "1:32" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0xc, .val_name = "1:64" },
    { .mask = 0xe, .mask_name = "WDTPS", .val = 0xe, .val_name = "1:128" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18c658_config4l[] = {
    { .mask = 0x02, .mask_name = "RES1", .val = 0x02, .val_name = "Reserved" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_device _pic18c658 = {
    .magic = DEV_MAGIC,
    .name = "PIC18C658",
    .arch = ARCH_18X,
    .support = SUPPORT_DASM | SUPPORT_PICSTART_PLUS | SUPPORT_PROMATE2,
    .memtech = MEMTECH_EPROM,
    .vdd = { .min = 2.500, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 12.750, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0xf00000,
    .memory_code = {
        .present = 1,
        .start = 0x0,
        .end  = 0x7fff,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x200000,
        .end  = 0x200007,
        .width = 1,
        .inc = 1,
        .mask = 0x0f,
        .init = 0xff
    },
    .memory_config = {
        .present = 1,
        .start = 0x300000,
        .end  = 0x300007,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_eeprom = { .present = 0 },
    .memory_dbgvec = { .present = 0 },
    .picstartplus = {
        .firstrev    = 0x020100,
        .algorithm   = 11,
        .overprogram = 3,
        .tries       = 25
    },
    .promate2 = {
        .firstrev = 0x051010,
        .os       = 0x11,
        .socket   = 0xd0,
        .part     = 0x0b
    },
    .config = {
        { .address = 0x300000, .name = "CONFIG1L", .bits = _pic18c658_config1l },
        { .address = 0x300001, .name = "CONFIG1H", .bits = _pic18c658_config1h },
        { .address = 0x300002, .name = "CONFIG2L", .bits = _pic18c658_config2l },
        { .address = 0x300003, .name = "CONFIG2H", .bits = _pic18c658_config2h },
        { .address = 0x300006, .name = "CONFIG4L", .bits = _pic18c658_config4l },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
