/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _pic18f2431_config1h[] = {
    { .mask = 0x80, .mask_name = "IESO", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x80, .mask_name = "IESO", .val = 0x80, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "FCMEN", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x40, .mask_name = "FCMEN", .val = 0x40, .val_name = "Enabled" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x0, .val_name = "LP" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x1, .val_name = "XT" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x2, .val_name = "HS" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x3, .val_name = "0011 EXT RC-CLKOUT on RA6" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x4, .val_name = "EC-CLKOUT on RA6" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x5, .val_name = "EC-Port on RA6" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x6, .val_name = "HS-PLL enabled freq=4xFosc1" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x7, .val_name = "EXT RC-Port on RA6" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x8, .val_name = "INT RC-Port on RA6,Port on RA7" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0x9, .val_name = "INT RC-CLKOUT on RA6,Port on RA7" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0xa, .val_name = "101X EXT RC-CLKOUT on RA6" },
    { .mask = 0xf, .mask_name = "OSC", .val = 0xc, .val_name = "11XX EXT RC-CLKOUT on RA6" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config2l[] = {
    { .mask = 0xc, .mask_name = "BODENV", .val = 0x0, .val_name = "4.5V" },
    { .mask = 0xc, .mask_name = "BODENV", .val = 0x4, .val_name = "4.2V" },
    { .mask = 0xc, .mask_name = "BODENV", .val = 0x8, .val_name = "2.7V" },
    { .mask = 0xc, .mask_name = "BODENV", .val = 0xc, .val_name = "Undefined" },
    { .mask = 0x2, .mask_name = "BODEN", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x2, .mask_name = "BODEN", .val = 0x2, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "PUT", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "PUT", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config2h[] = {
    { .mask = 0x20, .mask_name = "WINEN", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x20, .mask_name = "WINEN", .val = 0x20, .val_name = "Disabled" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x0, .val_name = "1:1" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x2, .val_name = "1:2" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x4, .val_name = "1:4" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x6, .val_name = "1:8" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x8, .val_name = "1:16" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0xa, .val_name = "1:32" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0xc, .val_name = "1:64" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0xe, .val_name = "1:128" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x10, .val_name = "1:256" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x12, .val_name = "1:512" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x14, .val_name = "1:1024" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x16, .val_name = "1:2048" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x18, .val_name = "1:4096" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x1a, .val_name = "1:8192" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x1c, .val_name = "1:16384" },
    { .mask = 0x1e, .mask_name = "WDTPS", .val = 0x1e, .val_name = "1:32768" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config3l[] = {
    { .mask = 0x20, .mask_name = "T1OSCMX", .val = 0x0, .val_name = "Legacy" },
    { .mask = 0x20, .mask_name = "T1OSCMX", .val = 0x20, .val_name = "Low Power" },
    { .mask = 0x10, .mask_name = "HPOL", .val = 0x0, .val_name = "PWM 1, 3, 5, and 7 are active low" },
    { .mask = 0x10, .mask_name = "HPOL", .val = 0x10, .val_name = "PWM 1, 3, 5, and 7 are active high" },
    { .mask = 0x8, .mask_name = "LPOL", .val = 0x0, .val_name = "PWM 0, 2, 4 and 6 are active low" },
    { .mask = 0x8, .mask_name = "LPOL", .val = 0x8, .val_name = "PWM 0, 2, 4 and 6 are active high" },
    { .mask = 0x4, .mask_name = "PWMPIN", .val = 0x0, .val_name = "PWM outputs drive active states upon RESET" },
    { .mask = 0x4, .mask_name = "PWMPIN", .val = 0x4, .val_name = "PWM outputs disabled upon RESET" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config3h[] = {
    { .mask = 0x80, .mask_name = "MCLRE", .val = 0x0, .val_name = "RE3 input enabled, MCLR disabled" },
    { .mask = 0x80, .mask_name = "MCLRE", .val = 0x80, .val_name = "MCLR enabled, RE3 input disabled" },
    { .mask = 0x10, .mask_name = "EXCLKMX", .val = 0x0, .val_name = "TMR0/T5CKI external clock input is multiplexed with RD0" },
    { .mask = 0x10, .mask_name = "EXCLKMX", .val = 0x10, .val_name = "TMR0/T5CKI external clock input is multiplexed with RC3" },
    { .mask = 0x8, .mask_name = "PWM4MX", .val = 0x0, .val_name = "PWM4 output muxed w/ RD5" },
    { .mask = 0x8, .mask_name = "PWM4MX", .val = 0x8, .val_name = "PWM4 output muxed w/ RB5" },
    { .mask = 0x4, .mask_name = "SSPMX", .val = 0x0, .val_name = "SPI not assigned" },
    { .mask = 0x4, .mask_name = "SSPMX", .val = 0x4, .val_name = "SCK/SCL, SDA/SDI and SDO are mux w/ RC5, RC4 and RC7 respectively." },
    { .mask = 0x1, .mask_name = "FLTAMX", .val = 0x0, .val_name = "FLTA input muxed with RD4" },
    { .mask = 0x1, .mask_name = "FLTAMX", .val = 0x1, .val_name = "FLTA input muxed with RC1" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config4l[] = {
    { .mask = 0x80, .mask_name = "BACKBUG", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "BACKBUG", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x4, .mask_name = "LVP", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x4, .mask_name = "LVP", .val = 0x4, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config5l[] = {
    { .mask = 0x8, .mask_name = "CP_3", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x8, .mask_name = "CP_3", .val = 0x8, .val_name = "Disabled" },
    { .mask = 0x4, .mask_name = "CP_2", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x4, .mask_name = "CP_2", .val = 0x4, .val_name = "Disabled" },
    { .mask = 0x2, .mask_name = "CP_1", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "CP_1", .val = 0x2, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "CP_0", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "CP_0", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config5h[] = {
    { .mask = 0x80, .mask_name = "CPD", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "CPD", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x40, .mask_name = "CPB", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "CPB", .val = 0x40, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config6l[] = {
    { .mask = 0x8, .mask_name = "WRT_3", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x8, .mask_name = "WRT_3", .val = 0x8, .val_name = "Disabled" },
    { .mask = 0x4, .mask_name = "WRT_2", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x4, .mask_name = "WRT_2", .val = 0x4, .val_name = "Disabled" },
    { .mask = 0x2, .mask_name = "WRT_1", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "WRT_1", .val = 0x2, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "WRT_0", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "WRT_0", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config6h[] = {
    { .mask = 0x80, .mask_name = "WRTD", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "WRTD", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x40, .mask_name = "WRTB", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "WRTB", .val = 0x40, .val_name = "Disabled" },
    { .mask = 0x20, .mask_name = "WRTC", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x20, .mask_name = "WRTC", .val = 0x20, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config7l[] = {
    { .mask = 0x8, .mask_name = "EBTR_3", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x8, .mask_name = "EBTR_3", .val = 0x8, .val_name = "Disabled" },
    { .mask = 0x4, .mask_name = "EBTR_2", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x4, .mask_name = "EBTR_2", .val = 0x4, .val_name = "Disabled" },
    { .mask = 0x2, .mask_name = "EBTR_1", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "EBTR_1", .val = 0x2, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "EBTR_0", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "EBTR_0", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _pic18f2431_config7h[] = {
    { .mask = 0x40, .mask_name = "EBTRB", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "EBTRB", .val = 0x40, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_device _pic18f2431 = {
    .magic = DEV_MAGIC,
    .name = "PIC18F2431",
    .arch = ARCH_18X,
    .support = SUPPORT_DASM | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.000, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 9.000, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0xf00000,
    .memory_code = {
        .present = 1,
        .start = 0x0,
        .end  = 0x3fff,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x200000,
        .end  = 0x200007,
        .width = 1,
        .inc = 1,
        .mask = 0x0f,
        .init = 0xff
    },
    .memory_config = {
        .present = 1,
        .start = 0x300000,
        .end  = 0x30000d,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_eeprom = {
        .present = 1,
        .start = 0x00,
        .end  = 0xff,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x200028,
        .end  = 0x200037,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .promate2 = {
        .firstrev = 0x061001,
        .os       = 0x11,
        .socket   = 0xb0,
        .part     = 0x4b
    },
    .icd2 = {
        .family = ICD2_FAMILY_18F,
        .efid   = 0x04,
        .famid  = 0x87,
        .debugexec = ICD2_DEBUGEXEC_DE18F_BANK02,
        .debugset = ICD2_DEBUGSET_18F
    },
    .config = {
        { .address = 0x300001, .name = "CONFIG1H", .bits = _pic18f2431_config1h },
        { .address = 0x300002, .name = "CONFIG2L", .bits = _pic18f2431_config2l },
        { .address = 0x300003, .name = "CONFIG2H", .bits = _pic18f2431_config2h },
        { .address = 0x300004, .name = "CONFIG3L", .bits = _pic18f2431_config3l },
        { .address = 0x300005, .name = "CONFIG3H", .bits = _pic18f2431_config3h },
        { .address = 0x300006, .name = "CONFIG4L", .bits = _pic18f2431_config4l },
        { .address = 0x300008, .name = "CONFIG5L", .bits = _pic18f2431_config5l },
        { .address = 0x300009, .name = "CONFIG5H", .bits = _pic18f2431_config5h },
        { .address = 0x30000a, .name = "CONFIG6L", .bits = _pic18f2431_config6l },
        { .address = 0x30000b, .name = "CONFIG6H", .bits = _pic18f2431_config6h },
        { .address = 0x30000c, .name = "CONFIG7L", .bits = _pic18f2431_config7l },
        { .address = 0x30000d, .name = "CONFIG7H", .bits = _pic18f2431_config7h },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
