/*
 * lplab - linux replacement for mplab
 *
 * (c) 2002-2003 Stephen Landamore <stephen@landamore.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include "devices.h"

static const struct pic_config_bit _ps500_config2h[] = {
    { .mask = 0x1, .mask_name = "WDT", .val = 0x0, .val_name = "Disabled-Controlled by SWDTEN bit" },
    { .mask = 0x1, .mask_name = "WDT", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config3h[] = {
    { .mask = 0x80, .mask_name = "MCLRE", .val = 0x00, .val_name = "MCLR disabled, RA5 input enabled" },
    { .mask = 0x80, .mask_name = "MCLRE", .val = 0x80, .val_name = "MCLR enabled, RA5 input disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config4l[] = {
    { .mask = 0x80, .mask_name = "BACKBUG", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "BACKBUG", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x0, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "STVR", .val = 0x1, .val_name = "Enabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config5l[] = {
    { .mask = 0x2, .mask_name = "CP_23", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "CP_23", .val = 0x2, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "CP_01", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "CP_01", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config5h[] = {
    { .mask = 0x80, .mask_name = "CPD", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "CPD", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x40, .mask_name = "CPB", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "CPB", .val = 0x40, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config6l[] = {
    { .mask = 0x2, .mask_name = "WRT_23", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "WRT_23", .val = 0x2, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "WRT_01", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "WRT_01", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config6h[] = {
    { .mask = 0x80, .mask_name = "WRTD", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x80, .mask_name = "WRTD", .val = 0x80, .val_name = "Disabled" },
    { .mask = 0x40, .mask_name = "WRTB", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "WRTB", .val = 0x40, .val_name = "Disabled" },
    { .mask = 0x20, .mask_name = "WRTC", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x20, .mask_name = "WRTC", .val = 0x20, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config7l[] = {
    { .mask = 0x2, .mask_name = "EBTR_23", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x2, .mask_name = "EBTR_23", .val = 0x2, .val_name = "Disabled" },
    { .mask = 0x1, .mask_name = "EBTR_01", .val = 0x00, .val_name = "Enabled" },
    { .mask = 0x1, .mask_name = "EBTR_01", .val = 0x1, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_config_bit _ps500_config7h[] = {
    { .mask = 0x40, .mask_name = "EBTRB", .val = 0x0, .val_name = "Enabled" },
    { .mask = 0x40, .mask_name = "EBTRB", .val = 0x40, .val_name = "Disabled" },
    { .mask_name = NULL }
};

static const struct pic_device _ps500 = {
    .magic = DEV_MAGIC,
    .name = "PS500",
    .arch = ARCH_18X,
    .support = SUPPORT_DASM | SUPPORT_PROMATE2 | SUPPORT_ICD2P | SUPPORT_ICD2D,
    .memtech = MEMTECH_EE,
    .vdd = { .min = 2.000, .max = 5.500, .nominal = 5.000 },
    .vpp = { .min = 9.000, .max = 13.250, .nominal = 13.000 },
    .eeprom_hexfile_offset = 0xf00000,
    .memory_code = {
        .present = 1,
        .start = 0x0,
        .end  = 0x3fff,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .memory_cal = { .present = 0 },
    .memory_id = {
        .present = 1,
        .start = 0x200000,
        .end  = 0x200007,
        .width = 1,
        .inc = 1,
        .mask = 0x0f,
        .init = 0xff
    },
    .memory_config = {
        .present = 1,
        .start = 0x300000,
        .end  = 0x30000d,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_eeprom = {
        .present = 1,
        .start = 0x00,
        .end  = 0xff,
        .width = 1,
        .inc = 1,
        .mask = 0xff,
        .init = 0xff
    },
    .memory_dbgvec = {
        .present = 1,
        .start = 0x200028,
        .end  = 0x200037,
        .width = 2,
        .inc = 2,
        .mask = 0xffff,
        .init = 0xffff
    },
    .promate2 = {
        .firstrev = 0x061023,
        .os       = 0x11,
        .socket   = 0xb0,
        .part     = 0x50
    },
    .icd2 = {
        .family = ICD2_FAMILY_18F,
        .efid   = 0x04,
        .famid  = 0x83,
        .debugexec = ICD2_DEBUGEXEC_DE18F_BANK00,
        .debugset = ICD2_DEBUGSET_18F
    },
    .config = {
        { .address = 0x300003, .name = "CONFIG2H", .bits = _ps500_config2h },
        { .address = 0x300005, .name = "CONFIG3H", .bits = _ps500_config3h },
        { .address = 0x300006, .name = "CONFIG4L", .bits = _ps500_config4l },
        { .address = 0x300008, .name = "CONFIG5L", .bits = _ps500_config5l },
        { .address = 0x300009, .name = "CONFIG5H", .bits = _ps500_config5h },
        { .address = 0x30000a, .name = "CONFIG6L", .bits = _ps500_config6l },
        { .address = 0x30000b, .name = "CONFIG6H", .bits = _ps500_config6h },
        { .address = 0x30000c, .name = "CONFIG7L", .bits = _ps500_config7l },
        { .address = 0x30000d, .name = "CONFIG7H", .bits = _ps500_config7h },
        { .name = NULL }
    },
};

/*
 * Local Variables:
 * c-basic-offset: 4
 * End:
 */
