/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef DIRECT_PRG_CONFIG_WIDGET
#define DIRECT_PRG_CONFIG_WIDGET

#include <qmainwindow.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qwidgetstack.h>
#include <qtimer.h>

#include "../ConfigWidget.h"

class hardware ;
class QRadioButton ;
class port ;
class QSlider ;

class DirectProgConfigWidget : public ConfigWidget
{
	Q_OBJECT
private:
  hardware *phard ;

  QComboBox *dev ;
  QComboBox *vpp ;
	QComboBox *vdd ;
	QComboBox *clock ;
	QComboBox *din ;
	QComboBox *dout ;
  //++Mirko!!
  QComboBox *drw ;
  //--Mirko!!

	QCheckBox *ivpp ;
	QCheckBox *ivdd ;
	QCheckBox *iclock ;
	QCheckBox *idin ;
	QCheckBox *idout ;
  //++Mirko!!
	QCheckBox *idrw ;
  //--Mirko!!

  QCheckBox *cbvpp;
  QCheckBox *cbvdd;
  QCheckBox *cbclock;
  QCheckBox *cbdatain;
  QCheckBox *cbdataout;
  //++Mirko!!
  QCheckBox *cbrw;
  //--Mirko!!

  QGroupBox * groupverif ;
  QPushButton *bstopverify ;
  QPushButton *bsendbits ;
  QTimer *timersendbits , *timerpolldataout ;

  /**  */
  QRadioButton *butpar, *butser;

  /** */
  QComboBox *parportlist, *serportlist ;
  QSlider *delay_slider ;

  /**  */

  //++Mirko!!
  void reconfigCombos(port* aport, int vp, int vd, int cl, int out, int in, int
          rw, int clkdelay );
  //--Mirko!!

  void send_bits( unsigned int d , int nbb) ;

  QComboBox *stdh ;
  QPushButton *stdsav ;
  /** A dummy programmer name which acts as a combobox title */
  const QString null_prog_name;

public:
  DirectProgConfigWidget(QWidget *parent);
  virtual ~DirectProgConfigWidget();

  /** Search the pin number pin in the string list list.
\return The index of the found pin , or -1 if not found
 */
  static int findPin(const QStringList &list, int pin);
  /** Convert a pin name to pin number
(a pin name must be a numeric decimal string  eventually followed by a blank and an arbitrary text) */
  static int pinNumber(const QString& pinname);
  /** Extract the pin name from the Pin number+pin name string */
  static const QString pinName(const QString &pinname);

public slots:
	virtual void doApply() ;
  /** Continuously send bits to programmer board */
  void sendBits();
  void slotvpp() ;
  void slotvdd() ;
  void slotclock() ;
  void slotdataout() ;

  //++Mirko!!
  void slotrw();
  void slotrwchange(const QString & string);
  //--Mirko!!

  void slotupdatedatain() ;
  /** No descriptions */
  void slotSendBits();
  /** No descriptions */
  void slotParDeviceSelected(bool  state);
  /** No descriptions */
  void slotSerDeviceSelected(bool  state);
  /** Changes pin/port  configuration when a new standard hardware is selected */
  void slotStdHardware(const QString& name) ;
  /** Creates a new hardware profile */
  void slotSaveHardware();
signals:
	void PrgConfigChanged() ;
} ;
#endif

