#include <vector>
#include <string>
#include <sstream>
using namespace std ;

#include "PicFactory.hh"

#include "pic12C508.hh"
#include "pic12C509.hh"
#include "pic12C671.hh"
#include "pic12C672.hh"

#include "pic16F73.hh"
#include "pic16F76.hh"
#include "pic16F83.hh"
#include "pic16F84.hh"
#include "pic16C83.hh"
#include "pic16C84.hh"
#include "pic16F874.hh"
#include "pic16F877.hh"
#include "pic16F628.hh"
#include "pic16F627.hh"
#include "pic16F872.hh"
#include "pic18F242.hh"
#include "pic18F252.hh"
#include "pic18F1220.hh"
#include "pic18F1320.hh"
#include "pic18F2220.hh"
#include "pic18F2320.hh"
#include "pic16F877a.hh"
#include "pic16F874a.hh"
#include "pic12F675.hh"
#include "pic16F630.hh"
#include "pic16F818.hh"
#include "pic16F819.hh"
#include "pic16F87.hh"
#include "pic16F627a.hh"
#include "pic16F628a.hh"
#include "pic16F648a.hh"
#include "pic16F688.hh"
#include "pic16F684.hh"
#include "pic16F636.hh"
#include "pic12F635.hh"

#include "pic16C745.hh"
#include "pic16C923.hh"
#include "pic16C61.hh"
#include "pic16C62.hh"
#include "pic16C65.hh"
#include "pic16C62A.hh"
#include "pic16C63A.hh"
#include "pic16C620.hh"
#include "pic16C621.hh"
#include "pic16C76.hh"
#include "pic16C773.hh"
#include "pic16C710.hh"
#include "pic16C711.hh"

// add here headers for new pic flavors

PicFactory * PicFactory::the_factory = 0 ;

PicFactory * PicFactory::factory()
{
		return  (the_factory == 0) ? ( the_factory = new PicFactory ) :  the_factory ;
}

//
// This method must be updated when a new pic driver is developed
//
PicFactory::PicFactory()
{
    // 12C family
    registerPic(new pic12C508) ;    
    registerPic(new pic12C509) ;        
    registerPic(new pic12C671) ;
    registerPic(new pic12C672) ;
    // 16F family
    registerPic(new pic16F73) ;
    registerPic(new pic16F76) ;
    registerPic(new pic16F83) ;
    registerPic(new pic16F84) ;
    registerPic(new pic16F874) ;
    registerPic(new pic16F877) ;
    registerPic(new pic16F872) ;
    registerPic(new pic16F628) ;
    registerPic(new pic16F627) ;
    registerPic(new pic16F877a) ;
    registerPic(new pic16F874a) ;
    registerPic(new pic16F630) ;
    registerPic(new pic16F818) ;
    registerPic(new pic16F819) ;
    registerPic(new pic16F87) ;
    registerPic(new pic16F627A) ;
    registerPic(new pic16F628A) ;
    registerPic(new pic16F648A) ;
    registerPic(new pic16F688) ;
    registerPic(new pic16F684) ;  // also 12F683
    registerPic(new pic16F636) ; 
      // 16C family
    registerPic(new pic16C83) ;
    registerPic(new pic16C84) ;
    registerPic(new pic16C745) ;
    registerPic(new pic16C923) ;
    registerPic(new pic16C61) ;
    registerPic(new pic16C62) ;
    registerPic(new pic16C65) ;
    registerPic(new pic16C62A) ;
    registerPic(new pic16C63A) ;
    registerPic(new pic16C620) ;
    registerPic(new pic16C621) ;
    registerPic(new pic16C76) ;
    registerPic(new pic16C773) ;
    registerPic(new pic16C710) ;
    registerPic(new pic16C711) ;
     // 18F family
    registerPic(new pic18F242) ;
    registerPic(new pic18F252) ;
    registerPic(new pic18F1220) ;
    registerPic(new pic18F1320) ;
    registerPic(new pic18F2220) ;
    registerPic(new pic18F2320) ;
     // 12F family (12Fxxx are in fact 14 bit pics)
    registerPic(new pic12F675) ;
    registerPic(new pic12F635) ; 

}

PicFactory::~PicFactory()
{
    std::vector<pic*>::iterator i ;

    for(i = piclist.begin() ; i != piclist.end() ; ++i)
        delete *i ;
}
//
// returns a pic flavor, or NULL if not found
//
pic* PicFactory::getPic(const char* picname)
{
    std::vector<pic*>::iterator i ;

    for(i = piclist.begin() ; i != piclist.end() ; ++i)
    {
        string nn((*i)->names()) ;
        std::istringstream st(nn) ;
        string n ;
        while( st >> n )
        	if(n == picname )
            		return (*i) ;
    }
    return 0 ; // not found
}
//
// May be used to add a new pic flavor after the construction
// of the pic factory    
void    PicFactory::registerPic(pic *p) 
{
    piclist.push_back(p) ;
}
//
// Returns a blank-separated list of all pic names
//
std::string   PicFactory::getPicNames()
{
    std::vector<pic*>::iterator i ;
    string  s ;
    
    for(i = piclist.begin() ; i != piclist.end() ; ++i)
    {     
        s += (*i)->names() ;
        s += " " ;
    }
    return s ;
}  

    
    
    
/** Convert a pic id to pic name
\return 
-A pic name if found, 
-an empty string if id == 0 or id == All one
-"?" if the device is not supported

 */
std::string PicFactory::findDeviceName(unsigned int id)
{
		if( id == 0 || id == 0x3FE0 || id == 0xFFE0 ) return "" ;

		std::vector<pic*>::iterator i ;

    for(i = piclist.begin() ; i != piclist.end() ; ++i)
    {
		   int idnum =  findID( (*i)->IDs(), id ) ;
			 if( idnum != -1 )
			 {
        string nn((*i)->names()) ;
        std::istringstream st(nn) ;
        string n ;
        for( ; idnum >= 0 && st >> n  ; --idnum ) ;
        return n ;
			 }
    }
    return "?" ;

}
/** Search for and id in the id string s
\return The index of the id if found, -1 if not found
 */
int PicFactory::findID(const char*s, unsigned int id)
{
	int n , k ;
	unsigned int i ;
	for( k=0 ; sscanf(s,"%x%n",&i,&n) == 1 ; s+=n, ++k)
	{
		if( i == id) return k ;
	}
	return -1 ;
}
