/***************************************************************************
                          Config.cpp  -  description
                             -------------------
    begin                : dim dc 21 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "direct_prog_config.h"

#include <kapplication.h>
#include <kconfig.h>

//++Mirko!!
void DirectProgrammerConfig::getCurrentPortConfig(QString &type, QString& port, int& vpp, int& vdd,
      int& clock, int& dataout, int& datain, int& drw, int& clkdelay)
//--Mirko!!
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("hardware") ;
    type = conf->readEntry("port-type", "parallel") ;
    if(type == "parallel" )
    {
      //++Mirko!!
			getParPortConfig(port, vpp, vdd, clock, dataout, datain, drw , clkdelay);
      //--Mirko!!
    }
    else if(type == "serial")  // serial port
    {
      //++Mirko!!
			getSerPortConfig(port, vpp, vdd, clock, dataout, datain, drw , clkdelay);
      //--Mirko!!
    }
}


//++Mirko!!
void DirectProgrammerConfig::setPortConfig(const QString &type, const QString& port, int vpp,
    int vdd, int clock, int dataout, int datain, int drw, int clkdelay )
//--Mirko!!
{
    KConfig *conf = kapp->config() ;
    conf->setGroup("hardware") ;
    conf->writeEntry("port-type",type) ;

    conf->setGroup(type) ;
    conf->writeEntry("device",port) ;
    conf->writeEntry("vpp" , vpp ) ;
    conf->writeEntry("vdd" ,vdd ) ;
    conf->writeEntry("clock" ,clock ) ;
    conf->writeEntry("datao" , dataout ) ;
    conf->writeEntry("datai" , datain ) ;
    //++Mirko!!
    conf->writeEntry("drw" , drw ) ;
    //--Mirko!!
    conf->writeEntry("clkdelay" , clkdelay ) ;
    conf->sync() ;
}


/** No descriptions */
//++Mirko!!
void DirectProgrammerConfig::getParPortConfig(QString &port, int &vpp, int& vdd, int &clock,
    int &dataout, int& datain, int&drw, int& clkdelay )
//--Mirko!!
{
    KConfig *conf = kapp->config() ;

    // tait's std
    conf->setGroup("parallel") ;
    clock     = conf->readNumEntry("clock",3) ;
    datain    = conf->readNumEntry("datai", 10 ) ;
    dataout   = conf->readNumEntry("datao", 2 ) ;
    //++Mirko!!
    //Default RW Line not used...
    drw       = conf->readNumEntry("drw", 25 ) ;  // 25 = GND
    //--Mirko!!
    vdd       = conf->readNumEntry("vdd", -4 ) ;
    vpp       = conf->readNumEntry("vpp", -5 ) ;
    clkdelay  = conf->readNumEntry("clkdelay", 0 ) ;
    port      = conf->readEntry("device", "/dev/parport0" ) ;
}


//++Mirko!!
void DirectProgrammerConfig::getSerPortConfig(QString &port, int &vpp, int& vdd, int &clock,
  int &dataout, int& datain, int& drw, int& clkdelay )
//--Mirko!!
{
    KConfig *conf = kapp->config() ;

    // JDMcp  std
    conf->setGroup("serial") ;
    clock     = conf->readNumEntry("clock",7) ;
    datain    = conf->readNumEntry("datai", 8 ) ;
    dataout   = conf->readNumEntry("datao", 4 ) ;
    //++Mirko!!
    //Default RW Line not used...
    drw       = conf->readNumEntry("drw", 5 ) ; // 5 = GND
    //--Mirko!!
    vdd       = conf->readNumEntry("vdd", 5 ) ;
    vpp       = conf->readNumEntry("vpp", 3 ) ;
    clkdelay  = conf->readNumEntry("clkdelay", 0 ) ;
    port      = conf->readEntry("device", "/dev/ttyS0" ) ;
}


/** Write informations about  known standard programming cards */
void DirectProgrammerConfig::writeStdHardwares()
{
//     int vpp, int vdd, int clock, int dataout, int datain, int drw
  setHardConfig("Tait classic","parallel", -5,-4,3,2,10,25 ,  0) ;
  setHardConfig("Tait 7405/7406","parallel", 5,4,-3,-2,-10,25 ,  0) ;
  setHardConfig("P16PRO40 classic","parallel", 5,4,-3,-2,-10,25 ,  0) ;
  setHardConfig("P16PRO40 7407","parallel", -5,-4,3,2,10,25,  0 ) ;
  setHardConfig("P16PRO40-VPP40 classic","parallel", 6,4,-3,-2,-10,25 ,  0) ;
  setHardConfig("P16PRO40-VPP40 7407","parallel", -6,-4,3,2,10,25 ,  0) ;
  setHardConfig("EPIC+ (You must disconnect 7407 pin 2)","parallel", -5,-4,3,2,10,25,  0 ) ;
  setHardConfig("JDM classic","serial", 3,5,7,4,8,5,  0 ) ;
  setHardConfig("JDM classic (delay 10)","serial", 3,5,7,4,8,5,  10 ) ;
  setHardConfig("JDM classic (delay 20)","serial", 3,5,7,4,8,5,  20 ) ;
  setHardConfig("PIC Elmer","serial", -3,5,-7,-4,-8,5 ,  0) ;
  setHardConfig("Velleman K8048","serial", -3,5,-7,-4,-8,5 ,  0) ;

  //++Mirko!!
  //Added by Mirko Panciri 10/03/2004...
  //Visit http://www.pic-tools.com
  //I have tested only the "Asix Piccolo" version...
  //I think the lines is the same of "Asix Piccolo Grande"...
  //Who have "Asix Piccolo Grande" contact panciom@libero.it for info / result of test!!!
  setHardConfig("Asix Piccolo","parallel",-6,-7,-5,-3,-10,-2,  0) ;
  setHardConfig("Asix Piccolo Grande","parallel",-6,-7,-5,-3,-10,-2,  0) ;
  //--Mirko!!
  setHardConfig("Propic2 Vpp-1","parallel",-5,-4,3,2,10,25,  0) ;
  setHardConfig("Propic2 Vpp-2","parallel",-6,-4,3,2,10,25,  0) ;
  setHardConfig("Propic2 Vpp-3","parallel",-7,-4,3,2,10,25,  0) ;
  setHardConfig("Myke's EL Cheapo","parallel",16, 25, -1,-17,13,25,  0) ;
  setHardConfig("EL Cheapo classic (not tested)","parallel",16, 25, 1,17,-13,25,  0) ;
  
}


//++Mirko!!
void DirectProgrammerConfig::setHardConfig(const QString& name, const QString &type, int vpp,
  int vdd, int clock, int dataout, int datain, int drw, int clkdelay)
//--Mirko!!
{
	KConfig *conf = kapp->config() ;
	QStringList h =  DirectProgrammerConfig::getStdHardwares() ;
	if( h.findIndex(name) == -1)
	{
    conf->setGroup("hardware") ;
		conf->writeEntry("std-programmers", h+=name) ;
	}
  conf->setGroup("hardware-"+name) ;
  conf->writeEntry("vpp" , vpp ) ;
  conf->writeEntry("vdd" ,vdd ) ;
  conf->writeEntry("clock" ,clock ) ;
  conf->writeEntry("datao" , dataout ) ;
  conf->writeEntry("datai" , datain ) ;
  conf->writeEntry("port-type" , type ) ;
  //++Mirko!!
  conf->writeEntry("drw" , drw ) ;
  //--Mirko!!
  conf->writeEntry("clkdelay" , clkdelay ) ;
}


/** Return the list of std known programmers */
QStringList DirectProgrammerConfig::getStdHardwares()
{
	KConfig *conf = kapp->config() ;
  conf->setGroup("hardware") ;
  return conf->readListEntry( "std-programmers") ;
}


/** Read the pin configuration for the hardware "name"
*/
//++Mirko!!
void DirectProgrammerConfig::getHardConfig(const QString& name, QString &type, int& vpp, int& vdd,
  int& clock, int& dataout, int& datain, int& drw, int& clkdelay)
//--Mirko!!
{
  KConfig *conf = kapp->config() ;
  conf->setGroup("hardware-"+name) ;

  // returned values are defaulted to Tait if the specified hardware does not exist
  vpp = conf->readNumEntry("vpp", -5 ) ;
  vdd = conf->readNumEntry("vdd", -4  ) ;
  clock = conf->readNumEntry("clock", 3 ) ;
  dataout = conf->readNumEntry("datao", 2  ) ;
  //++Mirko!!
  //Default RW line not used...
  drw = conf->readNumEntry("drw", 25  ) ;
  //--Mirko!!
  datain = conf->readNumEntry("datai", 10 ) ;
  type = conf->readEntry("port-type","parallel" ) ;
  clkdelay =  conf->readNumEntry("clkdelay", 0 ) ;
}
