/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "direct_programmer.h"

#include <sstream>
#include <qregexp.h>

#include <klocale.h>

#include "direct_prog_config.h"
#include "PicFactory.hh"
#include "memory.h"

DirectProgrammer::DirectProgrammer()
  : _pic(0)
{}

void DirectProgrammer::internalSetDevice(const QString &name)
{
  _pic = (PicFactory::factory())->getPic("p" + name.lower());
  Q_ASSERT(_pic);
}

QString DirectProgrammer::port() const
{
  QString type, port;
  int vpp, vdd, clock, dataout, datain, datarw, clkdelay;
  DirectProgrammerConfig::getCurrentPortConfig(type,port,vpp,vdd,clock,dataout,datain,datarw,clkdelay);
  return port;
}

bool DirectProgrammer::internalConnectDevice()
{
  QString type, port;
  int vpp, vdd, clock, dataout, datain, datarw, clkdelay;
  DirectProgrammerConfig::getCurrentPortConfig(type,port,vpp,vdd,clock,dataout,datain,datarw,clkdelay);
  _hard.setpins(type.latin1(),port.latin1(),vpp,vdd,clock,dataout,datain,datarw,clkdelay);
  if ( !_hard.isok() ) {
    setError(i18n("Cannot open port \"%1\".<br><br>.").arg(port));
    return false;
  }
  _pic->setPort(&_hard);
  // #### self-test ?
  return true;
}

bool DirectProgrammer::internalReadDeviceId(QString &name, uint &revision)
{
  unsigned int id = _pic->get_deviceID();
  name = _pic->deviceID2Name(id).c_str();
  revision = _pic->deviceID2Revision(id);
  return true;
}

bool DirectProgrammer::internalRead(Memory &memory)
{
  _pic->setMem(memory.data());
  _pic->read_pic();
  return true;
}

bool DirectProgrammer::internalErase()
{
  int res = _pic->erase_pic();
  if ( res==-16 ) {
    setError(i18n("Erase operation not supported."));
    return false;
  }
  return true;
}

bool DirectProgrammer::internalVerify(const Memory &memory)
{
  _pic->setMem(const_cast<Memory &>(memory).data());
  ostringstream out;
  int res = _pic->verify_pic(out); // #### FIXME
  QString s = out.str().c_str();
  log(Log::Information, s);
  if ( res!=0 ) {
    setError(_pic->error_msg(res)); // #### FIXME
    return false;
  }
  return true;
}

bool DirectProgrammer::internalProgram(const Memory &memory)
{
  _pic->setMem(const_cast<Memory &>(memory).data());
  ostringstream out;
  int res = _pic->program_pic(out); // #### FIXME
  QString s = out.str().c_str();
  log(Log::Information, s);
  if ( res!=0 ) {
    setError(_pic->error_msg(res)); // #### FIXME
    return false;
  }
  return true;
}
