/***************************************************************************
                          pic12F675.cc  -  description
                             -------------------
    begin                : dim nov 23 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pic12F675.hh"
//#include <iostream>
using namespace std ;

pic12F675::pic12F675()
{
    memaddr1 = 0x0 ; memaddr2 = 0x3FF ; // 1024 words
    idaddr1 = 0x2000 ; idaddr2 = 0x2003 ;
    eeaddr1 = 0x2100 ; eeaddr2 = 0x217F ;  // 128 bytes
    cfaddr1 = cfaddr2  = 0x2007 ;

}

/** For this device the OSCCAL and BG bits must be preserved before
erasing, and restored after erasing. */
int pic12F675::erase_pic()
{
    unsigned int cfw, BGbits, CALword , dummy = 0x3FFF ;

    // read OSCCAL  value   (location == 0x3FF
    pulseEngine("cdP,B,") ;
    for ( int x = memaddr1 ; x < memaddr2 ; x++ ) pulseEngine("k6,") ;  // goto last word
    pulseEngine("k4,R,",&CALword) ;
    //read BG bits ;
    pulseEngine("k0,S,k6,k6,k6,k6,k6,k6,k6,k4,R,", &cfw ) ;
    BGbits = cfw | 0x0FFF ;

    // bulk erase program memory and data memory
    pulseEngine("k0,S,k9w9000k11w9000", &dummy) ;
    // shutdown
    pulseEngine("bpcd" ) ;

    // reenter programming mode
    pulseEngine("cdP,B,") ;
    for ( int x = memaddr1 ; x < memaddr2 ; x++ ) pulseEngine("k6,") ;  // goto last word
    pulseEngine("k2,S,k8w7000",&CALword) ;  // reprogram calibration word
    pulseEngine("k0,S,k6,k6,k6,k6,k6,k6,k6,k2,S,k8w7000", &BGbits ) ;  // reprogram BG
    pulseEngine("bpcd" ) ;

    return 0 ;
}
/** No descriptions */
const char * pic12F675::names()
{
    return "p12f675 p12f629" ;
}
/** No descriptions */
const char * pic12F675::IDs()
{
    return "FC0 F80" ;
}
/** No descriptions */
const char * pic12F675::config_bits()
{
    return "BG[13-12]{Badgap calibration bits: They are preserved\nby PiKdev during programmation and bulk erase.}"
    " 0(11-9){Unimplemented: Read as '0'}"
    " /CPD[8]{1=Data memory not protected\n0=Data memory protected}"
    " /CP[7]{1=Program memory is not protected\n0=Pregram memory protected}"
    " BODEN[6]{1=BOD reset enabled\n0=BOD reset disabled}"
    " MCLRE[5]{1=/MCLR pin is /MCLR function\n0=/MCLR pin is alternate function}"
    " /PWRTE[4]{1=PWRT disabled\n0=PWRT enabled}"
    " WDTE[3]{1=WDT enabled\n0=WDT disabled}"
    " FOSC[2-0]"
  "{111=RC oscillator, CLKOUT on GP4\n"
  "110=RC oscillator, I/O on GP4\n"
  "101=INTOSC, CLCKOUT on GP4\n"
  "100=INTOSC, I/O on GP4\n"
  "011=EXTCLK, I/O on GP4\n"
  "001=XT oscillator, crystal on GP4\n"
  "000=LP oscillator, low power crystal on GP4} "
 ;
}
/** No descriptions */
int pic12F675::program_pic(ostream &out)
{
    int stat , n  ;
    int x;
    unsigned int chip ;


    erase_pic() ;
    stat = 0 ;

    pulseEngine("cdP,B,") ;

    // the last word (memaddr2 sddr) must not be programmed (osscal word)
    for ( x = memaddr1; x < memaddr2 ; x++ )
    {
        if( pulseEngine( "k4,F,V" , mem()+x) ) // do not overwrite correct data
        {
            n = pulseEngine("k2,S,k8w10000k4,F,V,", mem()+x) ;
            if( n != 0 )
            {
                stat |= 1 ;   // report failure
                pulseEngine("?", &chip) ; // get chip value
                warning(out,"Prog memory",x,chip,mem()[x]) ;
            }
        }
        pulseEngine("k6,") ;
    }
    pulseEngine("k6,") ;  // skip the last word

    // Program the EEPROM data area
    for( x = eeaddr1; x <= eeaddr2 ; x++ )
    {
        if( pulseEngine( "k4,F,V" , mem()+x) ) // do not overwrite correct data
        {
            n= pulseEngine("k3,S,k8w10000k5,f,v,", mem()+x) ;
            if( n != 0 )
            {
                stat |= 2 ;   // report failure
                pulseEngine("?", &chip) ;
                warning(out,"EEPROM data",x,chip,mem()[x]) ;
            }
        }
        pulseEngine("k6,") ;
    }

    // Program  ID configurations
    // enter configuration mode
    unsigned int dummy ;
    pulseEngine("k0,S,",&dummy);

    for( x = idaddr1; x <= idaddr2; x++ )
    {
        n= pulseEngine("k2,S,k8w10000k4,F,V,k6,", mem()+x) ;
        if( n != 0 )
        {
            stat |= 4 ;   // report failure
            pulseEngine("?", &chip) ;
            warning(out,"ID data",x,chip,mem()[x]) ;
        }
    }

    // program the config word : we must read it to preserve BG bits
    unsigned int cf  ;
    n=pulseEngine("k0,S,k6,k6,k6,k6,k6,k6,k6,k4,R,", &cf) ;
    cf &= 0x3000 ;
    if( cf != (mem()[cfaddr1] & 0x3000))
    {
        warning(out,"BG bits have been preserved",cfaddr1) ;
        stat |= 32 ;
    }
    cf |=  (mem()[cfaddr1] &  ~0x3000) ;
    pulseEngine("k2,S,k8w10000,k4,F,V,", &cf) ;
    if(n != 0)
    {
        stat |= 8 ;
        pulseEngine("?", &chip) ;
        warning(out,"Config fuses",cfaddr1,chip,cf) ;
    }


    /* shutdown sequence VPP off, VDD off, etc.. */
    pulseEngine("cdbp");

    return stat ;
}

const char *pic12F675::SPLocs()
{
  return "3FF=#FF0000{OSCCAL calibration word} "
  "[0-4]=#00FF00" ;
}

