/***************************************************************************
                          pic16C6_7_9xx.cc  -  description
                             -------------------
    begin                : mer jan 21 2004
    copyright            : (C) 2004 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pic16C6_7_9XX.hh"

pic16C6_7_9XX::pic16C6_7_9XX()
{
    memaddr1 = 0x0 ; memaddr2 = -1 ;  // Prog size is device dependant
    idaddr1 = 0x2000 ; idaddr2 = 0x2003 ;
    eeaddr1 = -1 ; eeaddr2 = -1 ;   // no EEPROM
    cfaddr1 = cfaddr2  = 0x2007 ;
}

int pic16C6_7_9XX::erase_pic()
{
    return 16 ;  /* 16 == NA */
}
/** No descriptions */
void pic16C6_7_9XX::read_pic()
{
    int x;
    unsigned int dummy = 0x3FFF;

    // enter programming  mode
    pulseEngine("cdP,B,") ;
    // Read the PIC's  program memory
    for ( x = memaddr1 ; x <= memaddr2 ; x++ )
        pulseEngine("k4,R,k6,", mem()+x ) ;

    // enter conf memory
    pulseEngine("k0,S,",&dummy) ;
    for ( x = idaddr1 ; x <= idaddr2 ; x++ )
        pulseEngine("k4,R,k6,", mem()+x ) ;

    // read Config Fuses
    pulseEngine("k6,k6,k6,k4,R,", mem()+cfaddr1) ;

    // power off sequence
    pulseEngine("cdpb")  ;

}

int pic16C6_7_9XX::program_pic(ostream& out )
{
    int stat , fail  ;
    int x;
    unsigned int dummy = 0x3FFF ;

    stat = 0 ;

    // programming init sequence
    pulseEngine("cdP,Bw100") ;

    for ( x = memaddr1; x <= memaddr2 ; ++x )
    {
        // speed up : enter prog process only if word is not already correct
        if(mem()[x] != INVALID && pulseEngine("k4,F,V" , mem()+x) != 0)
        {
            for(fail = 1 ; fail <= 25 ; ++fail)
                if(pulseEngine("k2,S,k8w100k14w5000k4,F,V", mem()+x) == 0)
                    break  ;
            if(fail > 25 )
            {
                pulseEngine("k4,R,", &dummy) ;
                stat |= 1 ;
                warning(out,"Prog memory", x, dummy, mem()[x]) ;
            }
            else // 3 x fail additionnal passes
            {
                for(fail *= 3 ; fail > 0 ; --fail)
                    pulseEngine("k2,S,k8,w100,k14,w5000", mem()+x) ;
            }
        }
        pulseEngine("k6") ;
    }

    // enter configuration memory
    pulseEngine("k0,S,",&dummy) ;

    for ( x = idaddr1; x <= idaddr2 ; ++x )
    {
        for(fail = 1 ; fail <= 25 ; ++fail)
            if(pulseEngine("k2,S,k8w150k14w5000k4,F,V", mem()+x) == 0)
                break  ;
        if(fail > 25 )
        {
            pulseEngine("k4,R,", &dummy) ;
            stat |= 4 ;
            warning(out,"ID memory", x, dummy, mem()[x]) ;
        }
        else // 3 x fail additionnal passes
        {
            for(fail *= 3 ; fail > 0 ; --fail)
                pulseEngine("k2,S,k8,w150,k14,w5000", mem()+x) ;
        }
        pulseEngine("k6") ;
    }

    pulseEngine("k6,k6,k6," ) ;

    for(fail = 1 ; fail <= 100 ; ++fail)
    {   // 100 cycles needed
        pulseEngine("k2,S,k8w100k14w5000",mem()+cfaddr1 )  ;
    }
    pulseEngine("k4,R,",&dummy ) ;
    if(dummy != mem()[cfaddr1] )
    {
        stat |= 8 ;
        warning(out,"Config fuses",cfaddr1,dummy,mem()[cfaddr1]  ) ;
    }

    pulseEngine("cdbp");

    return stat ;
}

int pic16C6_7_9XX::verify_pic(ostream &out)
{

    FragBuffer ram ;
    ram.swap(mem()) ;  // leave mem adresses unchanged

    int status = 0 , i ;


    read_pic() ;


    // compare  program memory
    for(i = memaddr1 ; i <= memaddr2 ; ++i)
    {
        if(mem()[i] != ram[i] && ram[i] != INVALID )
        {
            status |= 1 ;
            warning(out,"Prog memory",i ,mem()[i], ram[i]) ;
        }
    }

    // compare ID
    for(i = idaddr1 ; i <= idaddr2 ; ++i)
    {
        if(mem()[i] != ram[i] && ram[i] != INVALID)
        {
            status |= 4 ;
            warning(out,"ID data",i,mem()[i], ram[i]) ;
        }
    }

    // compare config fuses
    if(mem()[cfaddr1] != ram[cfaddr1] && ram[cfaddr1] != INVALID)
    {
        status |= 8 ;
        warning(out,"Config fuses",cfaddr1,mem()[cfaddr1], ram[cfaddr1]) ;
    }

    // restore ram buffer
    mem().swap(ram) ;  // restore old buffer

    return status;
}
