/*

Copyright 2004 Keith Baker (syzygy@pubnix.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <ctype.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>


#include "pic16F76.hh"




// ========================== 16F77/76	specific code ======================
//  16F76   TESTED : YES
// Seems to have trouble with ID bits, a program power up / down helps,
// but then config bits don't work.  I need config bits so I am leaving it
// alone for now...  odd.  Note that the manual says you just k0 and go to town // like all other burning, but the example it gives verifies with low power in
// between programming and id/config bits hence why I tired the powerdown (with // some delays for power supply smoothing).
// Device doesn't support erasing durring program
// Warnings are posted if their is a misconfigured bit that we will be unable
// to configure poroplerly.  A bit of optimization is done, we read first,
// compare, send + burn only if needed.  This speeds up small bits of code but
// will slow down a loaded chip.
// See pic16FXX.cc for a referance to the functions used in here.

pic16F76::pic16F76()  : pic16FXX()
{
    memaddr1 = 0x0; memaddr2 = 0x1FFF ; // 8K  words
    eeaddr1 = -1; eeaddr2 = -1 ;  // EEPROM : This device has none!
    idaddr1 = 0x2000 ; idaddr2 = 0x2003;
    cfaddr1 = cfaddr2  = 0x2007;
}

const char * pic16F76::names()
{
    return "p16f76 p16f77" ;
}
/** No descriptions */
const char * pic16F76::IDs()
{
	return "640 660" ;
}
/** No descriptions */
const char* pic16F76::config_bits()
{         // FIXME : (13-5 is wrong )
  return "1(13-5){Unimplemented: Read as '1'}"
      " BODEN[6]{1=Brown-Out Reset Enable, 0=Disable}"
      " 1(5){Unimplemented: Read as '1'}"
      " CPO[4]{1=Code Protection off, 0=Code Protection on}"
      " /PWRTEN[3]{0=Power Up Timer Enable, 1=Power Up Timer Disable}"
      " WDTEN[2]{1=Watchdog Timer Enable, 0=Watchdog Timer Disable}"
      " FOSC[1-0]{11=Resistor Capacitor oscillator\n"
      "10=High Speed Crystal Resonator\n"
      "01=Crystal Resonator\n"
      "00=Low Power Crystal.}";
}

int pic16F76::program_pic(ostream& out)
{
    int stat , n  ;
    int x;
    unsigned int word;
    stat = 0 ;
    unsigned int chip ;

    //Added this because this chip can't selectively erase blcoks...  we clean up the chip first
    erase_pic();

    // programming init sequence
    pulseEngine("cdP,B,", 0) ;

    // Program the PIC's main program code memory
    for ( x = memaddr1; x <= memaddr2 ; x++ )
      {
	//Read Memory for two purposes,
	//1) check to see if it needs to be altered
	//2) If it does need to be altered, it better be 0x3FFF
	// otherwise device wasn't erased
	pulseEngine("k4,R,", &word);
	if ((word != mem()[x]) && (mem()[x] != -1U) && (word != 0x3FFF))
	  {
	    warning(out,"Prog Device not Blank",x,chip,mem()[x]);
	  }
	else if((word != mem()[x]) && (mem()[x] != -1U))
	  {
	    n = pulseEngine("k2,S,k8w2000k14,k4,F,V", mem()+x) ;
	    if( n != 0 )
	      {
		stat |= 1 ;   // report failure
		pulseEngine("?",&chip) ;
		warning(out,"Prog memory",x,chip,mem()[x]) ;
	      }
	  }
	pulseEngine("k6,") ;	 // inc addr
      }

    // Program the ID configurations
    // enter configuration mode
    unsigned int dummy = 0x3FFF ;
    pulseEngine("k0,S,w2000" , &dummy);
    for( x = idaddr1; x <= idaddr2; x++ )
    {
        n= pulseEngine("k2,S,k8w2000k14,k4,F,V,", mem()+x) ;
        if( n != 0 )
        {
            stat |= 4 ;   // report failure
            pulseEngine("?",&chip) ;
            warning(out,"ID data",x,chip,mem()[x]) ;
        }
        pulseEngine("k6,") ;
    }

    // program the fuses

    n=pulseEngine("k6,k6,k6,k2,S,k8w2000k14,k4,F,V,",  mem() + cfaddr1) ;
    if(n != 0)
    {
        stat |= 8 ;
        pulseEngine("?",&chip) ;
        warning(out,"Config fuses",cfaddr1,chip,mem()[x]) ;
    }
    pulseEngine("k6,") ;  // why ???


    /* shutdown sequence VPP off, VDD off, etc.. */
    pulseEngine("cdbp" );

    return stat ;
}

int pic16F76::erase_pic()
{
  //Pic terminates erase itself...  just wait a while.
    pulseEngine("cdP,B,k9w400000;cdbp") ;
    return 0 ;
}
