/***************************************************************************
                          16F818.cc  -  description
                             -------------------
    begin                : sam fv 28 2004
    copyright            : (C) 2004 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pic16F818.hh"

pic16F818::pic16F818()
{
    memaddr1 = 0x0 ; memaddr2 = 0x3FF ; // 1024 words
    idaddr1 = 0x2000 ; idaddr2 = 0x2003 ;
    eeaddr1 = 0x2100 ; eeaddr2 = 0x217F ;  // 128 bytes
    cfaddr1 = cfaddr2  = 0x2007 ;
    //clk_delay = 10 ;

}

/** No descriptions */
const char * pic16F818::IDs()
{
    return "04C0" ;
}
/** No descriptions */
const char * pic16F818::names()
{
    return "p16f818" ;
}
/** No descriptions */
const char * pic16F818::config_bits()
{
    return "CP[13] {0=Program Memory not protected\n1=All memory protected}"
    " CCPMX[12]{1=CPP1 on RB2\n0=CPP1 on RB3}"
    " DEBUG[11]{1=In circuit degugger off\n0=in circuit degugger on}"
    " WRT[10-9]{11=Flash memory protection off\n10=0 to 1FF protected\n01=0 to 3FF protected\n11=0 to 5FF protected}"
    " CPD[8]{1=EE memory non protected\n0=EE memory protected}"
    " LVP[7]{1=RB3 has PGM function (LVP enabled)\n0=RB3 is digital I/O}"
    " BOREN[6]{1=BOR enabled\n0=BOR disabled}"
    " MCLRE[5]{1=RA5 is /MCLR\n0=RA5 is digital I/O}"
    " FOSC-2[4]{111=EXTRC, CLK0 on RA6\n110=EXTRC, RA6 is IO\n101=INTRC, CLK0 on RA6\n100=INTRC, RA6 is IO\n011=EXTCLK\n010=HS\n001=XT\n000=LP}"
    " /PWRTEN[3]{1=PWRT disabled\n0=PWRT enabled}"
    " WDTEN[2]{1=WDT enabled\n0=WDT disabled}"
    " FOSC[1-0]{111=EXTRC, CLK0 on RA6\n110=EXTRC, RA6 is IO\n101=INTRC, CLK0 on RA6\n100=INTRC, RA6 is IO\n011=EXTCLK\n010=HS\n001=XT\n000=LP}" ;
}


/** No descriptions */
int pic16F818::erase_pic()
{
    unsigned int word = 0x3FFF ;

    pulseEngine("cdP,B,k0,S,k31,w4000,cdbp", &word ) ;    // tprog3 = 2ms minimum
    return 0 ;

}
/** No descriptions */
int pic16F818::program_pic(ostream& o)
{
    int x;
    unsigned int dummy = 0x3FFF ;

    // I use the "prog only" algorithm (DS39603C p10) so I must erase the chip
    erase_pic() ;

    pulseEngine("cdP,B") ;

    // Program the PIC's main program code memory
    for ( x = memaddr1; x <= memaddr2 ;  )
    {
        pulseEngine("k2,S,k6,", &mem()[x++]) ;
        pulseEngine("k2,S,k6,", &mem()[x++]) ;
        pulseEngine("k2,S,k6,", &mem()[x++]) ;
        pulseEngine("k2,S,", &mem()[x++]) ;
        pulseEngine("k24w4000k23,k6,") ; // tprog1 = 2ms minimum
    }
    /* Program the EEPROM data area */
    for( x = eeaddr1; x <= eeaddr2 ;++x)
    {
        pulseEngine("k3,S,k24w4000k23,k6,", mem()+x) ;
    }

    // Program the ID configurations
    // enter configuration mode
    pulseEngine("k0,S,", &dummy );

    for( x = idaddr1; x <= idaddr2; )
    {
        pulseEngine("k2,S,k6,", &mem()[x++]) ;
        pulseEngine("k2,S,k6,", &mem()[x++]) ;
        pulseEngine("k2,S,k6,", &mem()[x++]) ;
        pulseEngine("k2,S,", &mem()[x++]) ;
        pulseEngine("k24w4000k23,k6,") ; // tprog1 = 2ms minimum
    }

    // program the fuses
    pulseEngine("k6,k6,k6,") ;
    for( x = cfaddr1 ; x <= cfaddr2 ; ++x)
    	pulseEngine("k2,S,k24w4000k23w100k6w100", &mem()[x] ) ;

    pulseEngine("w1000cdbp");

    return verify_pic(o) ;

}
