/***************************************************************************
                         pic18F242.cpp  -  description
                            -------------------
   begin                : mar jun 17 2003
   copyright            : (C) 2003 by Alain Gibaud
   email                : alain.gibaud@univ-valenciennes.fr

   Programming code mostly written by   Manwlis Giannos
   <mgiannos2000@yahoo.gr>

***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pic18F242.hh"

pic18F242::pic18F242()
{
    // word adresses !
    memaddr1 = 0x0/2 ; memaddr2 = 0x3FFF/2 ;
    idaddr1 = 0x200000/2 ;  idaddr2 = 0x200007/2 ;
    eeaddr1 = 0xF00000/2 ; eeaddr2 = 0xF000FF/2 ;
    cfaddr1 = 0x300000/2 ; cfaddr2 = 0x30000D/2 ;
}


/** No descriptions */
int pic18F242::verify_pic(ostream& out)
{
    FragBuffer ram ;
    ram.swap(mem()) ;  // leave mem adresses unchanged

    int status = 0 , i ;


    read_pic() ;


    // compare  program memory
    for(i = memaddr1 ; i <= memaddr2 ; ++i)
    {
        if(mem()[i] != ram[i] && ram[i] != INVALID )
        {
            status |= 1 ;
            warning(out,"Prog memory",i*2,mem()[i], ram[i]) ;
        }
    }

    // compare EEPROM data
    for(i = eeaddr1 ; i <= eeaddr2 ; ++i)
        if(mem()[i] != ram[i] && ram[i] != INVALID)
        {
            status |= 2 ;
            warning(out,"EEPROM",i*2,mem()[i], ram[i]) ;
        }

    // compare ID
    for(i = idaddr1 ; i <= idaddr2 ; ++i)
    {
        if(mem()[i] != ram[i] && ram[i] != INVALID)
        {
            status |= 4 ;
            warning(out,"ID data",i*2,mem()[i], ram[i]) ;
        }
    }

    // compare config fuses
    for( i = cfaddr1 ; i <= cfaddr2 ; ++i)
    {
        if(mem()[i] != ram[i] && ram[i] != INVALID)
        {
            status |= 8 ;
            warning(out,"Config fuses",i*2,mem()[i], ram[i]) ;
        }
    }

    // restore ram buffer
    mem().swap(ram) ;  // restore old buffer

    return status;
}

/** No descriptions */

int pic18F242::program_pic(ostream& out)
{
    unsigned int  data , w ;
    int  x ;

    erase_pic() ;

    // enter  Hi Voltage mode
    pulseEngine("cdpbP,B");
    // Memory code.

    // single panel programming technique, DS39576B page 12,13,14
    // step 1 & step 2   (1 panel write)

    pulseEngine("w300000k0,X0E3C,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E06,k0,X6EF6,k12,X0000,") ;
    // step 3

    pulseEngine("k0,X8EA6,k0,X9CA6,k0,X0E00,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E00,k0,X6EF6,") ;

    for(x=memaddr1 ; x <=  memaddr2 ; x+=4)
    {
        // step 4
        pulseEngine("k13,S,", &mem()[x]) ;
        pulseEngine("k13,S,", &mem()[x+1]) ;
        pulseEngine("k13,S,", &mem()[x+2]) ;
        pulseEngine("k15,S,", &mem()[x+3]) ;
        pulseEngine("d,C,c,C,c,C,c,Cw2000cw100X0000,") ;  // final sequence

        // "NULL" programming function, in fact increases
        //  PICs memory address pointer
        pulseEngine("k13,S,", &mem()[x]) ;
    }
    // end of memory code programming

    // EEProm DS39576B page  16,17
    // step 1
    pulseEngine("k0,X9EA6,k0,X9CA6,") ;

    int eex ;
    for( x = eeaddr1 , eex = 0 ; x <= eeaddr2 ; ++x )
    {
        // We have to do the job twice because
        // 2 bytes are packed in each word of hexfile !

        //  upper part of the word
        // step 2
        w = (eex & 0xFF) | 0x0E00 ;
        pulseEngine("k0,S,k0,X6EA9,", &w) ; // send address
        // step 3
        data =   (mem()[x]  >> 8) & 0xFF  ;  // send H byte
        w = data | 0x0E00  ;
        pulseEngine("k0,S,k0,X6EA8,", &w) ;
        // step 4
        pulseEngine("k0,X84A6,") ;
        // step 5
        pulseEngine("k0,X0E55,k0,X6EA7,k0,X0EAA,k0,X6EA7,") ;
        // step 6
        pulseEngine("k0,X82A6,") ;
        // step 7
        pulseEngine("k0,X0000,k0,X0000w5005k0,X94A6,") ;
        ++eex ;


        //  lower part of the word
        // step 2
        w = (eex & 0xFF) | 0x0E00 ;
        pulseEngine("k0,S,k0,X6EA9,", &w) ;
        // step 3
        data =   mem()[x] & 0xFF ; //  send L byte
        w = data | 0x0E00  ;
        pulseEngine("k0,S,k0,X6EA8,", &w) ;
        // step 4
        pulseEngine("k0,X84A6,") ;
        // step 5
        pulseEngine("k0,X0E55,k0,X6EA7,k0,X0EAA,k0,X6EA7,") ;
        // step 6
        pulseEngine("k0,X82A6,") ;
        // step 7
        pulseEngine("k0,X0000,k0,X0000w5005k0,X94A6,") ;
        ++eex ;
    }

    // Config Bits programming

    // step 1 & step 2   (1 panel write)

    for(x=cfaddr1, eex=0 ; x <=  cfaddr2 ; x++)
    {
        w = (eex & 0xFF) | 0x0E00 ;

        pulseEngine("k0,X0E3C,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E06,k0,X6EF6,k12,X0000,k0,X8EA6,"

                    "k0,X8CA6,k0,X0E30,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,S,k0,X6EF6,", &w) ;

        pulseEngine("k15,S,", &mem()[x]) ;
        pulseEngine("d,C,c,C,c,C,c,Cw2000cw10X0000,k0,X2AF6,") ;
        pulseEngine("k15,S,", &mem()[x]) ;  // final sequence
        pulseEngine("d,C,c,C,c,C,c,Cw2000cw10X0000") ;  // final sequence
        eex+=2;
    }
    // ID programming DS39576B page  18,19

    // step 1 & step 2   (1 panel write)

    pulseEngine("w300000k0,X0E3C,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E06,k0,X6EF6,k12,X0000,") ;
    // step 3

    pulseEngine("k0,X8EA6,k0,X9CA6,k0,X0E20,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E00,k0,X6EF6,") ;

    for(x=idaddr1 ; x <=  idaddr2 ; x+=4)
    {
        pulseEngine("k13,S,", &mem()[x]) ;
        pulseEngine("k13,S,", &mem()[x+1]) ;
        pulseEngine("k13,S,", &mem()[x+2]) ;
        pulseEngine("k15,S,", &mem()[x+3]) ;
        pulseEngine("d,C,c,C,c,C,c,Cw2000cw10X0000,") ;  // final sequence
        pulseEngine("k13,S,", &mem()[x]) ;
        // "NULL  programming function, in fact increases
        //  PICs memory address pointer
    }
    /* shutdown sequence VPP off, VDD off, etc.. */
    pulseEngine("cd,b,p,");

    return  verify_pic(out)  ;
}

/** No descriptions */


void pic18F242::read_pic()
{
    int x;
    unsigned int w, L, H ;

    // general read initialization
    pulseEngine("cdpbw300000,P,Bw300000");

    // Read the PIC's FLASH program memory

    pulseEngine("w300000k0,X0E00,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E00,k0,X6EF6w1000") ;

    for ( x = memaddr1 ; x <= memaddr2 ; x++ )
    {
        pulseEngine("R", &mem()[x]);
    }
    pulseEngine("cdw300000");

    // Read Pic's EEPROM memory
#if 0
// buggy code
    pulseEngine("k0,X9EA6,k0,X9CA6,k0,X0E00,k0,X6EA9,k0,X80A6,k0,X50A8,k0,X6EF5,w1000") ;
    for ( x = eeaddr1; x <= eeaddr2; x+=2 )
    {
        pulseEngine("r", &H );
        w = ( (x-eeaddr1+1) & 0xFF) | 0x0E00 ;
        pulseEngine("w3k0,S,k0,X6EA9,k0,X80A6,k0,X50A8,k0,X6EF5",
                    &w);
        pulseEngine("r", &L );
        mem()[eeaddr1+(x-eeaddr1)/2] = (L | (H<<8)) & 0xFFFF  ;
        w = ( (x-eeaddr1+2) & 0xFF) | 0x0E00 ;
        pulseEngine("w3k0,S,k0,X6EA9,k0,X80A6,k0,X50A8,k0,X6EF5",
                    &w);
    }
#else
	  int xb ;
    pulseEngine("k0,X9EA6,k0,X9CA6") ;
    for ( x = eeaddr1, xb = 0 ; x <= eeaddr2; ++x)
    {
				w = 0x0E00 | (xb & 0xFF) ;
			  pulseEngine("k0,S,k0,X6EA9,k0,X80A6,k0,X50A8,k0,X6EF5,", &w) ;
        pulseEngine("r", &H );
        ++xb ;
				w = 0x0E00 | (xb & 0xFF) ;
			  pulseEngine("k0,S,k0,X6EA9,k0,X80A6,k0,X50A8,k0,X6EF5,", &w) ;
        pulseEngine("r", &L );
        ++xb ;
        mem()[x] = (L | (H<<8)) ;

    }
#endif

    // Read Pic's Configuration bits memory


    pulseEngine("w300000k0,X0E30,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E00,k0,X6EF6");

    for ( x = cfaddr1; x <= cfaddr2; x++ )
    {
        pulseEngine("R", &mem()[x]);
    }

    // Read Pic's ID bits memory


    pulseEngine("w300000k0,X0E20,k0,X6EF8,k0,X0E00,k0,X6EF7,k0,X0E00,k0,X6EF6");

    for ( x = idaddr1; x <= idaddr2; x++ )
    {
        pulseEngine("R", &mem()[x]);
    }

    pulseEngine("cdpb");
}



/** No descriptions */
int pic18F242::erase_pic()
{
    pulseEngine("cdpbw300000P,B");

    pulseEngine("w100000k0;X0E3C;k0;X6EF8;k0;X0E00;k0;X6EF7;"
    				"k0;X0E04;k0;X6EF6;k12;X0080;k0;X0000;k0;X0000w5005");

    /* shutdown sequence VPP off, VDD off, etc.. */
    pulseEngine("cdpb");
    return 0;
}

/** No descriptions */
const char * pic18F242::names()
{
    return "p18f242 p18f248 p18f442 p18f448" ;
}
/** No descriptions */
const char * pic18F242::IDs()
{
	return "8480 8800 84A0 8820" ;
}
/** No descriptions */
const char * pic18F242::config_bits()
{
    return "0(15-14) /OSCEN[13] 0(12-11) FOSC[10-8] 0(7-0);"   // 300000
           "0(15-12) WDTPS[11-9] WDTEN[8] 0(7-4) BORV1[3] BORV2[2] BOREN[1] /PWRTE[0];"  // 300002
           "0(15-9) CCP2MX[8] 0(7-0);"   // 300004
           "0(15-8) /BKBUG[7] 0(6-3) LVP[2] 0[1] STVREM[0];" // 300006
           "CPD[15] CPB[14] 0(13-4) CP[3-0];"           // 300008
           "WDTD[15] WRTB[14] WTRC[13] 0(12-4) WTR[3-0];"  // 30000A
           "0(15) EBTRB[14] 0(13-4) EBTR[3-0];"  // 30000C
           ;
}


/** No descriptions */
unsigned int pic18F242::get_deviceID()
{

    unsigned int id ;
    // general read initialization
    pulseEngine("cdpb,P,B,w1000");

    // Read the PIC's ID word    (3FFFFE-3FFFFF)
    pulseEngine("k0,X0E3F,k0,X6EF8,k0,X0EFF,k0,X6EF7,k0,X0EFE,k0,X6EF6,w1000,R,cdpb", &id) ;
    // I forge a pseudo ID because some 16F devices have the same ID as a 18F device.
    return id | 0x8000 ;
}
