/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "editor.h"

#include <klocale.h>

const Editor::SlotData Editor::SLOT_DATA[] = {
  { "view_split_view", SLOT(slotAddView()), View, None },
  { "view_remove_view", SLOT(slotRemoveCurrentView()), View, AddViewCount },
  { "edit_undo", SLOT(slotUndo()), UndoRedo, UndoCount },
  { "edit_redo", SLOT(slotRedo()), UndoRedo, RedoCount },
  { "edit_cut", SLOT(slotCut()), Selection, SelectionCount },
  { "edit_copy", SLOT(slotCopy()), Selection, SelectionCount },
  { "edit_paste", SLOT(slotPaste()), Selection, None }, // #### FIXME
  { "edit_find", SLOT(slotSearch()), Search, None },
  { "edit_find_next", SLOT(slotFindNext()), Search, None }, // #### FIXME
  { "edit_find_previous", SLOT(slotFindPrevious()), Search, None }, // #### FIXME
  { "edit_replace", SLOT(slotReplace()), Search, None },
  { "edit_goto_line", SLOT(slotGotoLine()), Line, None },
  { "file_save", SLOT(slotSave()), Common, Modified },
  { "file_save_as", SLOT(slotSaveAs()), Common, None },
  { "file_print", SLOT(slotPrintDialog()), Print, None },
  { "view_line_numbers", SLOT(slotToggleLineNumbersOn()), Line, LineNumberOn },
  { "view_icon_borders", SLOT(slotToggleBorderOn()), Line, BorderOn },
  { "view_auto_wrap", SLOT(slotToggleWrapOn()), Line, AutoWrapOn },
  { "view_dos_eol", SLOT(slotToggleDosEOLOn()), Line, DosEolOn }, // #### FIXME (should be for HexEditor too)
  { "edit_comment", SLOT(slotComment()), Line, None },
  { "edit_uncomment", SLOT(slotUncomment()), Line, None },
  { 0, 0, Common, None }
};

Editor::Editor(const QString &ename, QWidget *parent, const char *name)
  : QWidget(parent, name), _name(ename)
{}

Editor::Editor(const PURL &url, QWidget *parent, const char *name)
  : QWidget(parent, name), _url(url)
{}

void Editor::slotSave()
{
  if ( _url.isEmpty() ) {
    _url = slotSaveAs();
    if ( _url.isEmpty() ) return;
  } else if ( !save(_url) ) return;
  setModified(false);
  emit guiChanged();
}

PURL Editor::slotSaveAs()
{
  QString filter = PURL::filter(fileType());
  PDirectory dir = (_url.isEmpty() ? PDirectory::home() : _url.directory());
  PURL purl = PURL::getSave(dir, filter, this, i18n("Save..."));
  if ( purl.isEmpty() ) return PURL();
  if ( !save(purl) ) return PURL();
  emit guiChanged();
  return purl;
}

bool Editor::slotLoad()
{
  Q_ASSERT( !_url.isEmpty() );
  if ( !load(_url) ) return false;
  setModified(false);
  slotStatusChanged();
  emit guiChanged();
  return true;
}
