/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef EDITOR_H
#define EDITOR_H

#include <qwidget.h>

class KPopupMenu;

#include "FNU.h"

// Generic editor
class Editor : public QWidget
{
  Q_OBJECT
public:
  Editor(const QString &ename, QWidget *parent, const char *name = 0);
  Editor(const PURL &url, QWidget *parent, const char *name = 0);

  enum Capability { Common = 1, Selection = 2, Search = 4,
                    Line = 8, Print = 16, View = 32, UndoRedo = 64 };
  enum ValueType { None, Modified, AddViewCount, UndoCount, RedoCount, SelectionCount,
                   LineNumberOn, BorderOn, AutoWrapOn, DosEolOn };
  struct SlotData {
    const char *action, *slot;
    Capability capability;
    ValueType type;
  };
  static const SlotData SLOT_DATA[];

  virtual int capabilities() const = 0;
  virtual uint value(ValueType type) const = 0;
  virtual PURL::FileType fileType() const = 0;
  virtual bool isModified() const = 0;
  virtual void setModified(bool modified) = 0;
  PURL url() const { return _url; }
  QString name() const { return _name; }
  virtual void setReadWrite(bool readWrite) = 0;
  virtual bool isReadWrite() const = 0;
  /** Set the line to be highlighted
    * @return true if OK, false if already selected
    */
  virtual bool selectLine(int line) = 0;
  virtual void updateBookmarksMenu(KPopupMenu *pop) = 0;
  virtual bool canConfigureEditor() const = 0;
  virtual void configureEditor() = 0;

public slots:
  bool slotLoad();
  void slotSave();
  PURL slotSaveAs();
  virtual void slotStatusChanged() = 0;

signals:
  void guiChanged();
  void statusChanged(const QString &text);
  void dropEventPass(QDropEvent *e);

protected:
  virtual bool load(const PURL &url) = 0;
  virtual bool save(const PURL &url) = 0;

private:
  QString _name;
  PURL    _url;
};

#endif
