/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef LOG_H
#define LOG_H

#include <qtextedit.h>

namespace Log
{
  enum ShowMode { DontShow, Show };
  enum Type { Command = 0, Error, Warning, Data, Debug, Information,
              Nb_LogTypes };

//-----------------------------------------------------------------------------
/** Class for displaying messages.
 * By default Log::Command and Log::Debug messages are not shown; they can be
 * enabled with the context menu.
 */
class Widget : public QTextEdit
{
  Q_OBJECT
public:
  Widget(QWidget *parent = 0, const char *name = 0);

  ShowMode showMode(Type type) const { return _modes[type]; }
  void setShowMode(Type type, ShowMode mode) { _modes[type] = mode; }

public slots:
  virtual void append(Type type, const QString &text);
  virtual void appendToLastLine(const QString &text);

protected:
  QPopupMenu *createPopupMenu(const QPoint &pos);

private slots:
  void toggleVisible(int i);

private:
  QPopupMenu *_popup;
  int         _id[Nb_LogTypes];
  ShowMode    _modes[Nb_LogTypes];

  struct LogData {
    const char *label, *color;
    ShowMode mode;
    const char *visibleConfig;
  };
  static const LogData DATA[Nb_LogTypes];
};

//-----------------------------------------------------------------------------
class Client
{
public:
  Client(Widget *log = 0) : _log(log), _parent(0) {}
  Client(Client *parent) : _log(0), _parent(parent) {}
  virtual ~Client() {}

  void setLogWidget(Widget *log) { _log = log; }
  void setParent(Client *parent) { _parent = parent; }
  Widget *logWidget() { return _log; }
  virtual void log(Type type, const QString &message);
  virtual void appendToLastLine(const QString &text);
  virtual void setError(const QString &message, ShowMode showBoxMode = DontShow);

  bool hasError() const { return !_error.isNull(); }
  QString error() const { return _error; }
  void resetError() { _error = QString::null; }

protected:
  Widget  *_log;
  Client  *_parent;
  QString  _error;
};

} // namespace

#endif
