/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MEMORY_H
#define MEMORY_H

#include <qstring.h>
#include <qvaluevector.h>
#include <qmap.h>
class QWidget;

#include "devices/devices.h"
#include "direct_prog/FragBuffer.hh"
class PURL;
class pic;
class MemorySpaceEditor;

class Memory
{
public:
  Memory();
  void setDevice(const QString &device);
  QString device() const { return _device; }

  bool isPresent(MemorySpace space) const { return _spec[space].present; }
  int hexFileStart(MemorySpace space) const;
  int hexFileEnd(MemorySpace space) const;
  int start(MemorySpace space) const { return _spec[space].start; }
  int end(MemorySpace space) const { return _spec[space].end; }
  uint length(MemorySpace space) const;
  int mask(MemorySpace space) const { return _spec[space].mask; }
  int width(MemorySpace space) const { return _spec[space].width; }
  QString label(MemorySpace space) const;
  uint nbBitsAddress(MemorySpace space) const;
  uint nbCharsAddress(MemorySpace space) const;
  uint nbBitsWord(MemorySpace space) const;
  uint nbCharsWord(MemorySpace space) const;
  QString specialLocations() const { return _specialLocations; }
  FragBuffer &data() { return _data; }
  bool isAddressInside(MemorySpace space, int address) const;
  uint *dataAtOffset(MemorySpace space, uint offset);
  uint *dataAtAddress(MemorySpace space, int address);
  const uint *dataAtOffset(MemorySpace space, uint offset) const;
  const uint *dataAtAddress(MemorySpace space, int address) const;
  void clearData();
  MemorySpaceEditor *createEditor(MemorySpace space, QWidget *parent);

  // #### this brain damage is for using "pic.hh" inherited devices
  class ConfigBit {
  public:
    int mask, value;
    QString mask_name, value_name, comment;
    bool readOnly;
  };
  class Config : public QValueVector<ConfigBit> {
  public:
    QString name;
  };
  typedef QMap<uint, Config> Configs;
  const Config &config(uint address) const { return _configs[address]; }
  static Configs processPikdevConfig(pic *p, uint start, uint nbCharsAddress);

  struct SpaceData {
    const char *name, *label;
    bool writable;
    uint nbDisplayLines;
  };
  static const SpaceData SPACE_DATA[Nb_MemorySpaces];
  struct ArchitectureData {
    const char *name;
    uint nbBitsPC, nbBitsProgWord, nbBitsDataWord;
  };
  static const ArchitectureData ARCHITECTURE_DATA[Nb_Architectures];

  /// Differents flavors of Intel HEX file formats
  enum HexFormat {
    IHX8M = 0,  ///< 8 bits "swapped" format
    IHX8S,
    IHX16,      ///< 16 bits format
    IHX32,      ///< 8 bit format with 32 bits addresses for PIC18XXXX
    Nb_HexFormats
  };
  static const char *HEX_FORMATS[Nb_HexFormats];
  bool saveHexFile(const PURL &url, HexFormat format, QString &error, QWidget *widget);
  bool loadHexFile(const PURL &url, QString &error, QWidget *widget);

private:
  QString _device;
  pic_device_mem_spec _spec[Nb_MemorySpaces];
  int _eepromHexFileOffset;
  FragBuffer _data;
  QString _specialLocations;
  Configs _configs;
  Architecture _arch;

  bool fetch_next_block(FragBuffer::iterator& start,  int *len);
  void write_hex_block(QTextStream &stream, int reclen, FragBuffer::iterator& data, HexFormat format);
  // Caution: this routine must be recoded for 16 bits devices. #### ??
  enum ReadHexError { NoError, OpenError, CRCError, UnrecognizedError };
  ReadHexError read_hex(const PURL &url, QWidget *widget);

  void setDevice(const pic &apic);
  void setDevice(const pic_device &device);
};

#endif
