/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "misc.h"

#include <ctype.h>
#include <unistd.h>

#include <qfontmetrics.h>

QCString toHex(uint value, uint nbChars)
{
  Q_ASSERT( nbChars>=1 && nbChars<=9 );
  // uint is 32 bits and apparently the compiler refuses to shift if 4*nbChars>=32...
  Q_ASSERT( nbChars>=8 || (value >> (4*nbChars))==0 );
  QCString format;
  format.setNum(nbChars);
  QCString s(nbChars);
  s.sprintf("%0" + format + "X", value);
  return s;
}

QString toHexLabel(uint value, uint nbChars)
{
  return "0x" + toHex(value, nbChars);
}

uchar fromHex(char c)
{
  c = toupper(c);
  return (c>='A' ? 10+c-'A' : c-'0');
}

uint fromHex(const QCString &s)
{
  ulong v = 0;
  for (uint i=0; i<s.length(); i++) {
    v *= 16;
    v += fromHex(s[i]);
  }
  return v;
}

void msleep(uint msec)
{
  if ( msec>=1000 ) {
    sleep(msec / 1000);
    msec %= 1000;
  }
  if (msec) usleep(msec * 1000);
}


uchar bin2bcd(uchar bin)
{
  char h = bin / 10;
  char l = bin % 10;
  return (h*16) + l;
}

uchar bcd2bin(uchar bcd)
{
  char h = bcd / 16;
  char l = bcd % 16;
  return (h*10) + l;
}

int maxCharWidth(const QFont &font, const QString &s)
{
  QFontMetrics fm(font);
  int w = 0;
  for (uint i=0; i<s.length(); i++)
    w = QMAX(w, fm.width(s[i]));
  return w;
}

int maxHexCharWidth(const QFont &font)
{
  return maxCharWidth(font, "0123456789ABCDEF");
}
