/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *   Copyright (C) 2002-2003 Stephen Landamore <stephen@landamore.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef _SERIAL_H
#define _SERIAL_H

#include <termios.h>

#include <qstring.h>
#include <qcstring.h>
#include <qfile.h>
#include <qstringlist.h>

#include "log.h"

class Device
{
public:
  Device() {}
  virtual ~Device() {}

  virtual bool needDrain() const = 0;
  virtual void serialInit(int speed, termios &info) const = 0;
};

class Port : public Log::Client
{
public:
  enum Type { Serial = 0, Parallel, Nb_Types };
  static const char *LABELS[Nb_Types];
  static const QStringList &deviceList(Type type);

  Port() {}
  virtual ~Port() {}
  virtual Type type() const = 0;
  virtual void close() = 0;
  virtual void log(Log::Type type, const QString &message);
  void logData(const QString &s);
};

class SerialPort : public Port
{
public:
  SerialPort();
  virtual ~SerialPort() { close(); }
  virtual Type type() const { return Serial; }

  static const QStringList &deviceList();

  bool init(const Device &device, const QString &port, int baud);
  bool setSpeed(int baud);
  void close();
  bool setFlowControl(bool on);
  bool getCts();
  bool setDtr(bool on);
  bool setRts(bool on);
  bool send(char c);
  bool send(const QCString &s);
  char getc(int timeout);
  QCString receive(uint length, int timeout); // in ms (negative is infinite)
  void flush();

private:
  static QStringList *_list;
  int     _handle;
  termios _initinfo, _info;
  const Device *_device;

  enum { BUFFER_MAX_LEN = 256 };
  char _rbuffer[BUFFER_MAX_LEN];
  int  _rlen, _rindex;

  static int baudToSpeed(int baud);
  static int probePort(const QString &device);
  bool select(int timeout);
};

class ParallelPort : public Port
{
public:
  ParallelPort() {}
  virtual ~ParallelPort() { close(); }
  virtual void close() {}
  virtual Type type() const { return Parallel; }

  static const QStringList &deviceList();

private:
  static QStringList *_list;

  static int probePort(const QString &device);
};

#endif
