/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROGRAMMER_H
#define PROGRAMMER_H

#include "log.h"
#include "device_list.h"
class ConfigWidget;
class QWidget;
class Memory;

class Programmer : public QObject, public Log::Client, public DeviceUser
{
  Q_OBJECT
public:
  static Programmer &current();
  static ConfigWidget *createConfigWidget(Type type, QWidget *parent);

  virtual ~Programmer() {}
  virtual QString port() const = 0;

  void setDevice(const QString &name);
  bool connectDevice();
  bool setupTarget();
  bool readDeviceId(QString &name, uint &revision);

  enum Action { Program = 0, Read, Erase, Verify, BlankCheck, Nb_Actions };
  static const char *ACTION_LABELS[Nb_Actions];
  bool doAction(Action action, Memory &memory);

signals:
  void totalSteps(int total);
  void progress(int step);

protected:
  static Programmer *_current;
  QString _name;

  virtual void init();
  Programmer() {}
  static Programmer *create(Type type);

  virtual void internalSetDevice(const QString &name) = 0;
  virtual bool internalConnectDevice() = 0;
  virtual bool internalSetupTarget() = 0;
  virtual bool internalReadDeviceId(QString &name, uint &revision) = 0;
  virtual bool internalRead(Memory &memory) = 0;
  virtual bool internalErase() = 0;
  virtual bool internalVerify(const Memory &memory) = 0;
  virtual bool internalProgram(const Memory &memory) = 0;
  virtual uint internalTotalSteps(Action action, const Memory &memory) const = 0;
};

#endif
